;------------------
; routine LCD 2lignes de 8 car  6 bits I/O : PortB 7 6 5 4 et PortA  0 1
; inclus routine delai4m.inc @4MHz
; programme simple pour decoder/ lire Telecommande Infrarouge Marantz 5200
;*******************************************************
; Quartz   = 4MHz
; Cycle    = 1/Fosc = 1uSec
;*******************************************************
#define miniSimDE	
#DEFINE debugging	

	ifdef miniSimDE
		p=16F84

;==========================================================================

;       Register Definitions

;==========================================================================

W                            EQU     H'0000'
F                            EQU     H'0001'

;----- Register Files------------------------------------------------------

INDF                         EQU     H'0000'
TMR0                         EQU     H'0001'
PCL                          EQU     H'0002'
STATUS                       EQU     H'0003'
FSR                          EQU     H'0004'
PORTA                        EQU     H'0005'
PORTB                        EQU     H'0006'
EEDATA                       EQU     H'0008'
EEADR                        EQU     H'0009'
PCLATH                       EQU     H'000A'
INTCON                       EQU     H'000B'

OPTION_REG                   EQU     H'0081'
TRISA                        EQU     H'0085'
TRISB                        EQU     H'0086'
EECON1                       EQU     H'0088'
EECON2                       EQU     H'0089'

;----- STATUS Bits --------------------------------------------------------

IRP                          EQU     H'0007'
RP1                          EQU     H'0006'
RP0                          EQU     H'0005'
NOT_TO                       EQU     H'0004'
NOT_PD                       EQU     H'0003'
Z                            EQU     H'0002'
DC                           EQU     H'0001'
C                            EQU     H'0000'

;----- INTCON Bits --------------------------------------------------------

GIE                          EQU     H'0007'
EEIE                         EQU     H'0006'
TOIE                         EQU     H'0005'
INTE                         EQU     H'0004'
RBIE                         EQU     H'0003'
TOIF                         EQU     H'0002'
INTF                         EQU     H'0001'
RBIF                         EQU     H'0000'

;----- OPTION Bits --------------------------------------------------------

NOT_RBPU                     EQU     H'0007'
INTEDG                       EQU     H'0006'
TOCS                         EQU     H'0005'
TOSE                         EQU     H'0004'
PSA                          EQU     H'0003'
PS2                          EQU     H'0002'
PS1                          EQU     H'0001'
PS0                          EQU     H'0000'

;----- EECON1 Bits --------------------------------------------------------

EEIF                         EQU     H'0004'
WRERR                        EQU     H'0003'
WREN                         EQU     H'0002'
WR                           EQU     H'0001'
RD                           EQU     H'0000'

;==========================================================================

;       RAM Definition

;==========================================================================

        __MAXRAM H'CF'
        __BADRAM H'07', H'50'-H'7F', H'87'

;==========================================================================

;       Configuration Bits

;==========================================================================

_CP_ON                       EQU     H'000F'
_CP_OFF                      EQU     H'3FFF'
_PWRTE_ON                    EQU     H'3FF7'
_PWRTE_OFF                   EQU     H'3FFF'
_WDT_ON                      EQU     H'3FFF'
_WDT_OFF                     EQU     H'3FFB'
_LP_OSC                      EQU     H'3FFC'
_XT_OSC                      EQU     H'3FFD'
_HS_OSC                      EQU     H'3FFE'
_RC_OSC                      EQU     H'3FFF'
;=============

EEPROM EQU 0x2100
#define bank0	bcf STATUS,RP0
#define bank1     bsf STATUS,RP0

	ELSE

		ERRORLEVEL -302
		Processor  16F84
        include <..\COMMON\p16f84_.inc>
	ENDIF
	
	Radix           DEC
	EXPAND
	__CONFIG   _CP_OFF & _WDT_OFF & _PWRTE_ON & _XT_OSC

;*****************************************************************************
; Equates, I/O, vars
;*****************************************************************************
RESET_V        	EQU     0x0000          ; Address of RESET Vector
ISR_V          	EQU     0x0004          ; Address of Interrupt Vector
;OSC_FREQ       	EQU     4000000      ; Oscillator Frequency is 4 MHz Quartz
OSC_FREQ       	EQU     3580000      ; Oscillator Frequency is 3.58MHz Cermique resonator
; afficheur LCD en mode 4 bits
LCD_DATA_TRIS	EQU TRISB	; LCD DATA			Port B 
LCD_DATA	EQU PORTB	; LCD DATA 11,12,13,14  bits 4,5,6,7
LCD_CTRL_TRIS	EQU TRISA	; LCD CTRL 			Port A  
LCD_CTRL	EQU PORTA	; LCD CTRL 4,5,6   	bits 0,1,2
	IFDEF minisimDE
LCD_E       	EQU 2		; LCD: Ligne 6 de commande de controle de l'afficheur
LCD_RW         	EQU 1		; LCD: Ligne 5 de Lecture/Ecriture de l'afficheur
LCD_RS         	EQU 0		; LCD: Ligne 4 de selection de l'afficheur-> RA0
	else
LCD_E          	EQU 1		; LCD: Ligne 6 de commande de controle de l'afficheur
LCD_RS         	EQU 0		; LCD: Ligne 4 de selection de l'afficheur-> RA0
	endif
LINE1          	EQU  0x080   ; Set display to line 1 character 0
LINE2          	EQU  0x0C0   ; Set display to line 2 character 0

;-----------------------
#define	DRDY	PORTB,0	; <- 	12 de AD7715
#define	DOUT    PORTB,1	; <-	13 de AD7715
#define	DIN 	PORTB,2	; ->    14 de AD7715
#define	CLCK 	PORTB,3	; ->     1 de AD7715
; CS au zero		  ->   4 de AD7715
; RESET,DVdd,AVdd	       au +5V
; RefIn-,AIn-,AGnd au           Zero Volt
; Refin+ sur ref 2,500V	
#define LED     	PORTA,3	; ->	led de controle
#define SERIAL_TX	PORTA,4	; -> 	transmission RS232
;---------------

	IFDEF minisimDE
#DEFINE	RAMstart   0x0C		; pour 16F84
IndexP          EQU 	RAMstart
Cpt1            EQU 	RAMstart+1     ; A counter, used multiple places
count1          EQU	RAMstart+2
count2          EQU	RAMstart+3
count3          EQU	RAMstart+4
LCDtmp          EQU	RAMstart+5
LCDtmp1         EQU	RAMstart+6
; variable pour convertion binaire --> BCD 16bits
bdixmil         EQU 	RAMstart+10
bmille          EQU 	RAMstart+11
bcent		EQU 	RAMstart+12
bdix		EQU 	RAMstart+13
bun		EQU 	RAMstart+14
btmp		EQU 	RAMstart+15
; variables pour calcul 16 bits
valx		EQU 	RAMstart+16 
val1		EQU 	RAMstart+18  ; MSB puis LSB
val2		EQu	Ramstart+20
AdcNow	        EQU	RamStart+22    
Moyenne	        EQU	RamStart+24
CptV            EQU	RamStart+26
;---
TREG           EQU	RamStart+28
RCount         EQU	RamStart+29
RBuffer        EQU	RamStart+30
ECount         EQU	RamStart+31
EBuffer        EQU	RamStart+32
CCount         EQU	RamStart+33
CBuffer        EQU	RamStart+34
drapeau        EQU     RAMstart+35
 
;drapeau:2= choix output 0=Rs232 1=LCD
;drapeau:1=1 format XXXX.X   
;drapeau:0 pour effacer zero inutile
LastRam EQU     RAMstart+36

	else
	 cblock     0x0C
IndexP  
Cpt1   
count1  
count2  
count3 
LCDtmp  
LCDtmp1  
; variable pour convertion binaire --> BCD 16bits
bdixmil  
bmille     
bcent	
bdix	
bun	
btmp
CptV   
; variables pour calcul 16 bits
valx:2	
val1:2	
val2:2		
AdcNow:2	  
Moyenne:2
TREG	
RCount 
RBuffer  
ECount   
EBuffer 
CCount  
CBuffer 
drapeau   
;drapeau:2= choix output 0=Rs232 1=LCD
;drapeau:1=1 format XXXX.X   
;drapeau:0 pour effacer zero inutile
LastRam  
	endc
	endif

	IF ( (LastRam) > 0x4F )
		ERROR   "Attention debordement zone RAM"
	ENDIF
;**************************************************************
; Program start
;**************************************************************
	ORG     RESET_V                 ; RESET vector location
RESET   
	GOTO    DEBUT
;**************************************************************
; This is the Periperal Interrupt routine. Should NOT get here
;**************************************************************
	ORG     ISR_V                   ; Interrupt vector location
INTERRUPT       BCF     STATUS, RP0     ; Select bank 0
                nop
                nop
		GOTO    INTERRUPT

;=============================================================================
; Table message to display: kept right up front to avoid crossing 0xff boundary
;=============================================================================

TABLE_ST       
	  IFDEF miniSimDE
               ADDLW 0xFF      ;-1      car PCL increment sur instruction MOVWF !!!
          ENDIF
               MOVWF  PCL  
          
MSG_InitAD 
		RETLW   'I'
                RETLW   'N'
		RETLW   'I'
		RETLW   'T'
		RETLW   ' '
		RETLW   'A'
		RETLW   'D'
                RETLW   'C'
                 RETLW   ' '
		RETLW   0x00

MSG_TREG
                RETLW   'T'
		RETLW   'R'
		RETLW   'E'
		RETLW   'G'
		RETLW   '='
		RETLW   0x00
                            
MSG_OK      
                RETLW   'O'
		RETLW   'K'
		RETLW   '='
		RETLW   0x00
MSG_PB        
                RETLW   'P'
		RETLW   'R'
		RETLW   'O'
		RETLW   'B'
		RETLW   'L'
		RETLW   'E'
		RETLW   'M'
		RETLW   'E'
		RETLW   0x00
MSG_AutoCal      
                RETLW   'A'
                RETLW   'U'
		RETLW   'T'
                RETLW   'O'
                RETLW   '-'
                RETLW   'C'
                RETLW   'A'
                RETLW   'L'
		RETLW   13
		RETLW   10	
		RETLW   0x00
MSG_Mesure      
                RETLW   'V'
                RETLW   'A'
		RETLW   'L'
                RETLW   'E'
                RETLW   'U'
                RETLW   'R'
		RETLW   '='
		RETLW   0x00
MSG_NB
                RETLW   'N'
		RETLW   'B'
		RETLW   '='
		RETLW   0x00


;*****************************************************************************
; Initialize processor registers
;*****************************************************************************
DEBUT:                                   ; POWER_ON Reset (Beginning of program)
		CLRF    STATUS          ; Do initialization, Select bank 0
		CLRF    INTCON          ; Clear int-flags, Disable interrupts
		CLRF    PCLATH          ; Keep in lower 2KByte
		CLRF    PORTA           ; ALL PORT outputs should output Low.
		CLRF    PORTB

		CLRF	valx             ;MSB
		CLRF	valx+1           ;LSB
		CLRF	val2             ;MSB
		CLRF	val2+1           ;LSB
		CLRF	val1
		CLRF	val1+1
		CLRF 	AdcNow
		CLRF	AdcNow+1
		CLRF	CptV
		CLRF	CptV+1
		CLRF	drapeau
	
		bank1
        	MOVLW   0x00           ;all outputs
        	MOVWF   TRISA
		MOVLW   0x03           ; RB-7-6-5-4-3-2 =outputs  RB-0-1= inputs
		MOVWF   TRISB
		MOVLW	0x07    	; Option register setup:
					; RB Pullups
					; TMR0 fed from internal clock
					; Assign prescaler to TMR0
					; use 1/256 prescaling ratio
		MOVWF	OPTION_REG

		bank0
		CALL    LCD_Init         ; Initialize LCDisplay
		
;============================
Pres
       
       	bCf LED          ; led on
	call LCD_Cls
	clrf Cpt1
	MOVLW	LINE1	;
	CALL	LCD_Cde		; Set to first line
Pres1

      movf Cpt1,w
	CALL read_EE
	incf Cpt1,f
	ANDLW	0x0FF		; test sur (W)
	BTFSC	STATUS, Z	; zero= fin de message ?
	GOTO	Pres2  	
	Call LCD_Putchar
	goto Pres1
	
Pres2	
        ;bsf drapeau,2
    	bsf LED          ; eteint led
	MOVLW	LINE2			;
	CALL	LCD_Cde		; Set to first line
	movlw EE1-EE0
	MOVWF Cpt1
Pres3
	movf Cpt1,w
	CALL read_EE
	incf	Cpt1,f
	ANDLW	0x0FF		; test sur (W)
	BTFSC	STATUS, Z	; zero= fin de message ?
	GOTO Pres4  	
	Call LCD_Putchar
	goto Pres3
Pres4
	call delay_2_5s
    	bCf LED          ; eteint on
	call LCD_Clear1
	MOVLW	LINE1			;
	CALL	LCD_Cde		; Set to first line
	movlw EE2-EE0
	MOVWF Cpt1
Pres5
	movf Cpt1,w
	CALL read_EE
	incf	Cpt1,f
	ANDLW	0x0FF		; test sur (W)
	BTFSC	STATUS, Z	; zero= fin de message ?
	GOTO	Pres6  	
	Call LCD_Putchar
	goto Pres5
Pres6
	IFDEF minisimDE 
       nop
         call delay_1s
        else
        call delay_2_5s
        endif
    

		call LCD_Clear2
		call LCD_Clear1
        	bsf LED          ; eteint led
	  
        	call InitAD7715	
        	
		call delay_2_5s
		bcf LED          
		call delay_2_5s
		bsf LED 
		
		call AutoCalibration

		call LCD_Clear1
		MOVLW   LINE1
		CALL    LCD_Cde       
		MOVLW   MSG_AutoCal    
		CALL    TABLE_MSG      

		call delay_2_5s
		bcf LED          
		call delay_2_5s
		bsf LED 
		call delay_2_5s
		bcf LED        
	        call delay_2_5s
               

		call LCD_Clear2
		MOVLW   LINE2
		CALL    LCD_Cde         
		MOVLW   MSG_TREG    
		CALL    TABLE_MSG      
		movf TREG,W
		call bcd3
		call delay_1s

Boucle_ICI			
		call LCD_Clear1
		MOVLW   LINE1
		CALL    LCD_Cde         
		MOVLW   MSG_Mesure      
		CALL    TABLE_MSG     

        ifdef debugging
;#define  valeur 2049
;                movlw valeur >> 8       ; MSB
;                movwf AdcNow
;                movlw valeur % 256      ; LSB
;                movwf AdcNow+1
  

	movlw 8			; MSB
	movwf	RCount
L_2
	bcf CLCK
	bcf STATUS,C		; C=0
	btfsc PORTB,1
	bsf STATUS,C			; C=1
	RLF AdcNow,F 	 	; rotation  gauche via carry
	bsf CLCK
	Decfsz RCount,F  	
        Goto L_2

	movlw 8			; LSB
	movwf	RCount
L_3
	bcf CLCK
	bcf STATUS,C		; C=0
	btfsc PORTB,1	
	bsf STATUS,C		; C=1
	RLF AdcNow+1,F  	; rotation  gauche via carry
	bsf CLCK
	Decfsz RCount,F 		
        Goto L_3
	call delay_25us














        ELSE
		call Lecture_AD7715
      
        ENDIF
	
        	movf AdcNow,w	 ;MSB
		movwf val1
                movf AdcNow+1,w	 ; LSB
		movwf val1+1
		call bcd5V
	       call delay_1s
	        
		bsf LED      
		
		call LCD_Clear2
		MOVLW   LINE2
		CALL    LCD_Cde         
		MOVLW   MSG_NB      
		CALL    TABLE_MSG    
                INCF	CptV+1,F	        ; incremente LSB
		BTFSC	STATUS,Z	; si debordement LSB=0=256
		incf	CptV,F	; incremente MSB
		movf CptV,w	
		movwf val1
	        movf CptV+1,w	
		movwf val1+1
		call bcd5V
		call delay_1s
                
             ;   movlw 4         ; bit 2 !
             ;   xorwf  drapeau,f
                
              	bcf LED 
              	
                movlw 13
                call Send_Char
                movlw 10
                call Send_Char
		goto Boucle_ICI
	
;*****************************************					
; Sous-programmes specifiques AD7715  
;*****************************************
Envoie_Cde
	bsf CLCK
	 movwf	EBuffer
	 Movlw      8            ;correspond  un byte de 8 bits
       Movwf      ECount     ;place dans RCount
Envoie_1 
        Bcf             CLCK
        Rrf             EBuffer,F  ;shift droite dans carry
        Btfsc           STATUS,C    ;si carry 0 alors saute
        Goto            Envoie_2         ;carry =1
        BCF             DIN
        Goto            Envoie_3
Envoie_2 
        BSF             DIN
Envoie_3 
        Call            delay_25us  
        BSF             CLCK  
        Call            delay_25us  
        Decfsz          ECount,F   ;ECount =ECount -1
        Goto            Envoie_1
	call delay_100us
     	return

;-----------------------
InitAD7715
	call LCD_Clear1
	MOVLW   LINE1
	CALL    LCD_Cde       
	MOVLW   MSG_InitAD           ; Point to Command messag
	CALL    TABLE_MSG       ; Display the message
	movlw 0x20
	call Envoie_Cde
	movlw 0xFF
	call Envoie_Cde
	movlw 0xFF
	call Envoie_Cde
	movlw 0xFF
	call Envoie_Cde
	movlw 0xFF
	call Envoie_Cde
	movlw 0x20
	call Envoie_Cde
	movlw 0x00
	call Envoie_Cde
 	movlw 0x28
	call Envoie_Cde
	Call Lecture_Reg
	movwf TREG			; test TREGister doit etre null
	andlw 0xFF
	btfsc STATUS,Z
	goto InitAD_2
	MOVLW   MSG_OK
	goto InitAD_3
InitAD_2
	MOVLW   MSG_PB           ; Point to Command message
InitAD_3
	CALL    TABLE_MSG       ; Display the message
	return

;----------------------------------
AutoCalibration
	movlw 0x10			; mode autocal
	call Envoie_Cde
	movlw 0x44
	call Envoie_Cde
	movlw 0x10			; mode Normal
	call Envoie_Cde
	movlw 0x04
	call Envoie_Cde
	return
;----------------------------
Lecture_AD7715
	movlw 0x38			; mode lecture DATA 16 bits
	call Envoie_Cde
	bsf CLCK
	clrf AdcNow
	clrf AdcNow+1
Lect_1	btfsc DRDY             ; attendre etat DRDY
	goto Lect_1

	movlw 8			; MSB
	movwf	RCount
Lect_2
	bcf CLCK
	call delay_25us
	bcf STATUS,C		; C=0
	btfsc DOUT
	bsf STATUS,C			; C=1
	RLF AdcNow,F 	 	; rotation  gauche via carry
	bsf CLCK
	call delay_25us
	Decfsz RCount,F  	
        Goto Lect_2

	movlw 8			; LSB
	movwf	RCount
Lect_3
	bcf CLCK
	call delay_25us
	bcf STATUS,C		; C=0
	btfsc DOUT	
	bsf STATUS,C		; C=1
	RLF AdcNow+1,F  	; rotation  gauche via carry
	bsf CLCK
	call delay_25us
	Decfsz RCount,F 		
        Goto Lect_3
	call delay_25us
	return
;-------------------------------
Lecture_Reg
	bsf CLCK	
	movlw 8			; LSB
	movwf	RCount
Lecture_1
	bcf CLCK               ; clock=0
	call delay_25us
	bcf STATUS,C           ; C=0
	btfsc DOUT	
	bsf STATUS,C           ; C=1
	RLF TREG,F             ; rotation  gauche via carry
	bsf CLCK               ; clock=1
	Decfsz RCount,F
        Goto Lecture_1
	return
;--------------------------

;**************************************************************
; Envoi de message via table de message dans la zone programme
;**************************************************************
TABLE_MSG
		MOVWF   Cpt1         ; Point to the first char. we want
LOOP_WR         
		MOVF   Cpt1,W         ; Renew the pointer
		CALL    TABLE_ST        ; Initiate table lookup
		XORLW   0x00            ; Is this the terminating char?
		BTFSC   STATUS,Z
		GOTO    END_WR
		CALL    Choix_Out     ;LCD_Putchar      ; Actually put the char on display
		INCF    Cpt1,f	; Point to the next char
		GOTO    LOOP_WR         ; Do next character
END_WR
		RETURN
;=========================
; rev 26 janv 2003 lcd4c.inc
; fichiers lis:   delais_b.inc  lcd4c.h
;	routines en mode 4 Bits avec afficheur 2*16  base de HD77480
;	LCD_Init : initialisation de l'afficheur
;	LCD_Putchar : envoi un caractere ( contenu dans w ) sur l'afficheur
;	LCD_Cde  : envoi une commande ( contenue dans w)  l'afficheur
;	LCD_Cls  : efface l'afficheur
;	LCD_Clear1 : efface la premiere ligne de l'afficheur
;	LCD_Clear2 : efface la deuxieme ligne de l'afficheur
;	LCD_Home : renvoi le curseur en haut a gauche
;	LCD_Line1 : renvoi le curseur au debut de la ligne 1
;	LCD_Line2 : renvoi le curseur au debut de la ligne 2
;	pour positionner le curseur sur la ligne 1 :	movlw 0x80+offset	call LCD_Cde
;	pour positionner le curseur sur la ligne 2 : 	movlw 0xC0+offset	call LCD_Cde
;
;	variables a definir dans le programme principal:
;	LCDtmp,LCDTmp1			; LCD
;
;	constantes a definir dans le programme principal:
;	LCD_DATA		EQU PORTB	; LCD: Port ou est branche le bus de donnee ( bits 4  7 )
;	LCD_DATA_TRIS	EQU TRISB	; LCD: Port ou est branche le bus de donnee ( bits 4  7 )
;	LCD_CTRL		EQU PORTA	; LCD: Port ou est branche le bus de commande
;	LCD_CTRL_TRIS	EQU TRISA	; LCD: Port ou est branche le bus de commande
;	LCD_E			EQU 1		; LCD: Ligne 6 de commande de controle de l'afficheur
;not used LCD_RW		EQU 1		; LCD: Ligne 5 de Lecture/Ecriture de l'afficheur
;	LCD_RS		EQU 0		; LCD: Ligne 4 de selection de l'afficheur
;
;	IMPORTANT : ce programme est optimis --> ne pas changer l'ordre des commandes


LCD_Cls  
	movlw	D'01'			; effacage de l'ecran
	call	LCD_Cde
	goto	delay_2_5ms

;-----------------
LCD_Clear1				; effacage de la 1ere ligne
	movlw	D'16'			
	movwf	LCDtmp1
	movlw	LINE1			; 0x80
	call	LCD_Cde
	call	delay_2_5ms
LCDc111
	movlw	' '
	call	LCD_Putchar
	decfsz	LCDtmp1,f
	goto	LCDc111
LCD_Line1	movlw	LINE1
	call	LCD_Cde
	goto	delay_2_5ms
;--------------

LCD_Clear2				; effacage de la deuxieme ligne
	movlw	D'16'		
	movwf	LCDtmp1
	movlw	LINE2			; 0xC0
	call	LCD_Cde
	call	delay_2_5ms
LCDcl21
	movlw	' '
	call	LCD_Putchar
	decfsz LCDtmp1,f
	goto	LCDcl21
	
LCD_Line2
	movlw	LINE2
	call	LCD_Cde
	goto	delay_2_5ms

LCD_Home
	movlw	D'02'
	call	LCD_Cde
	goto delay_2_5ms

LCD_Putchar
	bank0				; rajout 7/03/03 
	movwf	LCDtmp
	call	delay_1ms
	movlw   0x0f
	andwf   LCD_DATA,F
	movf    LCDtmp,w
	andlw   0xF0
	iorwf   LCD_DATA,F
	IFDEF minisimDE
	bcf     LCD_CTRL, LCD_RW
	nop
	endif
	bsf     LCD_CTRL, LCD_RS
	nop
	bsf     LCD_CTRL, LCD_E
	call		delay_25us
	bcf     LCD_CTRL, LCD_E
	call		delay_25us
	movlw   0x0f
	andwf   LCD_DATA,F
	swapf   LCDtmp,W
	andlw   0xF0
	iorwf   LCD_DATA,F
	nop
	bsf     LCD_CTRL, LCD_E
	call		delay_25us
	bcf     LCD_CTRL, LCD_E
	call		delay_25us
	return

LCD_Init
	bcf     LCD_CTRL, LCD_E
	nop
	bcf     LCD_CTRL, LCD_RS
	nop
	IFDEF minisimDE
	bcf     LCD_CTRL, LCD_RW
	endif
	call 	delay_2_5ms
	movlw	0x0f
	andwf	LCD_DATA,F
	movlw	0x030			; Commande pour interface 4 bits
	iorwf	LCD_DATA,F
	bank1
	movlw   0x0F
	andwf   LCD_DATA_TRIS,W
	movwf   LCD_DATA_TRIS		; Port en sortie
	bank0
	bsf     LCD_CTRL, LCD_E
	nop
	bcf     LCD_CTRL, LCD_E
	call 	delay_2_5ms
	bsf     LCD_CTRL, LCD_E
	nop
	bcf     LCD_CTRL, LCD_E
	call 	delay_2_5ms		
	bsf     LCD_CTRL, LCD_E
	nop
	bcf     LCD_CTRL, LCD_E
	call 	delay_100ms		; tempo 100ms , remplace l'attente Wait_Busy de l'afficheur
	movlw   0x0f
	andwf   LCD_DATA,F
	movlw   0x020           ; Commande pour interface 4 bits
	iorwf   LCD_DATA,F
	bsf     LCD_CTRL, LCD_E
	nop
	bcf     LCD_CTRL, LCD_E
	movlw   0x028		; Envoi de la commande Fonction 4-bit , Font, Nombre de lignes
	call    LCD_Cde
	movlw   0x008		; display off
	call    LCD_Cde
	movlw   0x00C		; display on (0x00C sans curseur, 0x00E avec , 0x00F clign. )
	call    LCD_Cde
	movlw   0x006
	
LCD_Cde
	movwf   LCDtmp
	IFDEF minisimDE
	call 	delay_100ms		; tempo 100ms , remplace l'attente Wait_Busy de l'afficheur
	endif
	movlw   0x0f
	andwf   LCD_DATA,F
	movf    LCDtmp,w          
	andlw   0xF0
	iorwf   LCD_DATA,F
	IFDEF minisimDE
	bcf     LCD_CTRL,LCD_RW
	nop
	ENDIF
	bcf     LCD_CTRL,LCD_RS
	nop
	bsf     LCD_CTRL,LCD_E
	nop
	bcf     LCD_CTRL,LCD_E
	movlw   0x0f
	andwf   LCD_DATA,F
	swapf   LCDtmp,W
	andlw   0xF0
	iorwf   LCD_DATA,F
	bsf     LCD_CTRL,LCD_E
	nop
	bcf     LCD_CTRL,LCD_E
	return
;-----------------------
;special pour routine d'attente 4800 bauds 
; .58 pour 4800 bauds  3.58MHz
; .67 pour 4800 bauds  4MHZ
delay_RS
        IF OSC_FREQ == 4000000 
          MOVLW  67      
        ENDIF
        IF OSC_FREQ == 3580000
           MOVLW  58      
        ENDIF
         MOVLW  58 
         MOVWF           count1	; 1000000/4800=208S    
         DECFSZ          count1,F	; 1 cycle si pas de saut
         GOTO            $-1		; 2cycles
         Return				; 2 cycles 


delay_RS_Half
        IF OSC_FREQ == 4000000 
          MOVLW  38     
        ENDIF
        IF OSC_FREQ == 3580000
           MOVLW  25      
        ENDIF
         MOVLW           25            ;  pour 4800 bauds  3.58MHz
         MOVWF           count1	; < 100S
         DECFSZ          count1,F 
         GOTO            $-1		
         Return			

;------------------------
delay           movlw   0x001       
                movwf   count1     
                movlw   0x001     
                movwf   count2    
                movlw   0x020       
                movwf   count3    
delay1          decfsz  count3,f    
                goto    $-1          
                decfsz  count2,f  
                goto    $-3           
                decfsz  count1,f      
                goto    $-5          
        return            
;--------------

delay_25us      movlw   0x01     
                movwf   count1
                movlw   0x01
                movwf   count2
                movlw   0x05
                movwf   count3
                goto    delay1
        
delay_100us     movlw   0x01    
                movwf   count1
                movlw   0x01
                movwf   count2
                movlw   20
                movwf   count3
                goto    delay1
        
delay_200us     movlw   0x01    
                movwf   count1
                movlw   0x01
                movwf   count2
                movlw   40
                movwf   count3
                goto    delay1
        
delay_1ms       movlw   0x01       
                movwf   count1
                movlw   0x01
                movwf   count2
                movlw   200
                movwf   count3
                goto    delay1

delay_2_5ms       movlw   0x01       
                movwf   count1
                movlw   0x02
                movwf   count2
                movlw   243
                movwf   count3
                goto    delay1


delay_10ms       movlw   0x01      
                movwf   count1
                movlw   8
                movwf   count2
                movlw   205
                movwf   count3
                goto    delay1
		
delay_100ms     movlw   0x01        
                movwf   count1
                movlw   79
                movwf   count2
                movlw   0x01
                movwf   count3
                goto    delay1

delay_500ms	movlw 2
		movwf count1	
		movlw 135
		movwf count2
		movlw 106
		movwf count3
		goto delay1
        
delay_1s      	movlw   4          
                movwf   count1
                movlw   14
                movwf   count2
                movlw   56
                movwf   count3
                goto   delay1
                
delay_2_5s      movlw   8          
                movwf   count1
                movlw   161
                movwf   count2
                movlw   218
                movwf   count3
                goto   delay1

delay_10s     movlw   31       
                movwf   count1
                movlw   133
                movwf   count2
                movlw   51
                movwf   count3
                goto   delay1
;-----------------------

; variable pour convertion binaire --> BCD 16bits
;bcent	EQU 	RAMstart+32
;bdix		EQU 	RAMstart+33
;bun		EQU 	RAMstart+34
;btmp		EQU 	RAMstart+35
;btmp1	EQU 	RAMstart+36
;val1		EQU 	RAMstart+37		; variable pour stockage valeur 16 bits
;val2		EQU 	RAMstart+39
;===============================================

bcd3	movwf  btmp		; affichage octet --> decimal 3 digits ( de 0 a 255 )
	movlw  '0'
        movwf  bcent
	movwf  bdix
bcd31	movlw  100
        subwf  btmp,w
	bnc    bcd32
	movwf  btmp
	incf   bcent,f
	goto   bcd31
bcd32   movlw  10
	subwf  btmp,w
	bnc    bcd33
	movwf  btmp
	incf   bdix,f
	goto   bcd32
bcd33	movlw  '0'
	addwf  btmp,w
	movwf  bun
	movf   bcent,w
	  call Choix_Out
	movf   bdix,w
	  call Choix_Out
	movf   bun,w
	  call Choix_Out
	return

; affichage word --> decimal 5 digits , de 0 a 65535 
; rev 9 sept 04  avec blanc sur les chiffres non significatifs utilise drapeau:0 
; et raitement special XXXX.X si drapeau:1=1
bcd5V   bCf drapeau,0
	movf	val1,w
	movwf	valx
	movf	val1+1,w
	movwf	valx+1

	swapf  val1,w	
	iorlw  0xf0
	movwf  bmille 
	addwf  bmille,f 
	addlw  0xE2 
	movwf  bcent 
	addlw  0x32 
	movwf  bun 
	movf   val1,w 
	andlw  0x0F 
	addwf  bcent,f 
	addwf  bcent,f 
	addwf  bun,f 
	addlw  0xE9 
	movwf  bdix 
	addwf  bdix,f 
	addwf  bdix,f 
	swapf  val1+1,w 
	andlw  0x0F 
	addwf  bdix,f 
	addwf  bun,f 
	rlf    bdix,f 
	rlf    bun,f 
	comf   bun,f 
	rlf    bun,f 
	movf   val1+1,w 
	andlw  0x0F 
	addwf  bun,f 
	rlf    bmille,f 
	movlw  0x07 
	movwf  bdixmil 
        movlw  0x0A
Lb1: 	addwf  bun,f 
	decf   bdix,f 
	btfss  STATUS,C
	goto   Lb1 
Lb2:	addwf  bdix,f 
	decf   bcent,f 
	btfss  STATUS,C
	goto   Lb2 
Lb3:	addwf  bcent,f 
	decf   bmille,f 
	btfss  STATUS,C
	goto   Lb3 
Lb4:	addwf  bmille,f 
        decf   bdixmil,f 
	btfss  STATUS,C
        goto   Lb4 

	movf   bdixmil,w
        BTFSC STATUS,Z
        BTFSC drapeau,0
        goto Lb40a
        movlw ' '
        goto Lb40b
Lb40a:  bsf drapeau,0
	addlw  '0'

Lb40b:  call Choix_Out
	movf   bmille,w
        BTFSC STATUS,Z
        BTFSc drapeau,0
        goto Lb41a
        movlw ' '
        goto Lb41b
Lb41a:  bsf drapeau,0
	addlw  '0'
Lb41b:  call Choix_Out  

	movf   bcent,w
        BTFSC STATUS,Z
        BTFSc drapeau,0
        goto Lb42a
        movlw ' '
        goto Lb42b
Lb42a:  bsf drapeau,0
	addlw  '0'
Lb42b:  call Choix_Out  

	movf   bdix,w
        BTFSC STATUS,Z
        BTFSc drapeau,0
        goto Lb43a
        BTFSC drapeau,1         ; affichage ..0.0 
        goto Lb43a
        movlw ' '
        goto Lb43b
Lb43a:  bsf drapeau,0
	addlw  '0'
Lb43b:  call Choix_Out  

        BTFSS drapeau,1       ; affichage ...0.0 
        goto Lb44             ; si drapeau ......X. = 1
	movlw	'.'             	; sinon ....0
	call Choix_Out
Lb44:	movf   bun,w
	addlw  '0'
        call Choix_Out
	movlw	' '
        call Choix_Out
	movf valx,w
	movwf val1
	movf valx+1,w
	movwf	val1+1
        BCF drapeau,1
	return
;---------
Choix_Out
        btfsC drapeau,2         ; si bit 2  zero choix=RS232 sinon LCD
        goto Send_Char
        goto  LCD_Putchar
  
;--------------
Send_Char
	 movwf	        CBuffer
	 Movlw           8           ;correspond  un byte de 8 bits
         Movwf           CCount         ;place dans RsCount
         Bcf             SERIAL_TX       ;bit start  0
         Call            delay_RS           ;wait valeur 1bit 4800 Bauds
         Rrf             CBuffer,F        ;shift droite dans carry
         Btfsc           STATUS,C        ;si carry 0 alors saute
         Goto            $+3             ;carry =1
         Bcf             SERIAL_TX       ;Tx =0
         Goto            $+2
         Bsf             SERIAL_TX       ;Tx =1
         Call            delay_RS
         Decfsz          CCount,F       ;RsCount =RsCount -1
         Goto            $-8
         Bsf             SERIAL_TX       ;bit de stop  1
        Call            delay_RS           ;byte envoy
  	return

;****************************************************************************		
read_EE	

	movwf   EEADR           ; defini offset addresse
	bank1			; 16F84 !!!!
	bsf     EECON1,RD       ; mode lecture
	bank0			; 16F84 !!!!
	movf    EEDATA,W       	; recupere lecture
	return


	ORG 0x2100
      ;"1234567890123456"
EE0 DT "MPLAB5.50 16F84",0
EE1 DT "AD7715zd.asm   ",0
EE2 DT "RS232 et LCD   ",0  
EE3 DT "19 juillet 05  ",0
   

		END
