;*******************************************************
; Quartz   = 4MHz
; Cycle    = 1/Fosc = 1uSec
;*******************************************************
#define miniSimDE

	include <p16f84_.inc>

	ifdef miniSimDE
	p=16F84
	else
	ERRORLEVEL      -302
	Processor       16F84
	endif
	Radix           DEC
	EXPAND
	__CONFIG   _CP_OFF & _WDT_OFF & _PWRTE_ON & _XT_OSC

;*****************************************************************************
; Equates, I/O, vars
;*****************************************************************************
RESET_V         EQU     0x0000          ; Address of RESET Vector
ISR_V           EQU     0x0004          ; Address of Interrupt Vector
OSC_FREQ        EQU     D'4000000'      ; Oscillator Frequency is 4 MHz

;	constantes a definir dans le programme principal:

; afficheur LCD en mode 4 bits
LCD_DATA_TRIS	EQU TRISB	; LCD DATA		Port B
LCD_DATA	EQU PORTB	; LCD DATA 11,12,13,14  bits 4,5,6,7
LCD_CTRL_TRIS	EQU TRISA	; LCD CTRL 		Port A
LCD_CTRL	EQU PORTA	; LCD CTRL 4,5,6   	bits 0,1,2
	ifdef miniSimDE
LCD_E           EQU 2		; LCD: Ligne de commande de controle de l'afficheur
LCD_RW          EQU 1		; LCD: Ligne de Lecture/Ecriture de l'afficheur
LCD_RS          EQU 0		; LCD: Ligne de selection de l'afficheur-> RA0
	else
LCD_E           EQU 1		; LCD: Ligne de commande de controle de l'afficheur
LCD_RS          EQU 0		; LCD: Ligne de selection de l'afficheur-> RA0
	endif

; PORTA bits
Capteur         EQU  0  ; RA0
; misc.
LINE1           EQU     0x080   ; Set display to line 1 character 0
LINE2           EQU     0x0C0   ; Set display to line 2 character 0

#DEFINE	Led_Hecto       3       ; led entre RA3 et +5V 1=eteint 0=allum
#DEFINE	Trx             PORTA,4
#DEFINE	RAMstart        0x0C	; pour 16F84
#define 	Tpd1            43      ; impuls pour 100 M
#define 	Tpd2            434     ; impuls pour 1Km

;Quartz 4MHz
;RA0 pour la Led
;RA1 pour le buzzer
;
;RBO pour la selection d un temps
;RB1 pour un autre temps
;RB2 idem
;RB3 pour un mode programme Lent
;RB4 Rapide
;
;En mode Mtronome simple :
;La led doit s'allumer 20mSec toutes les x secondes suivant la slection RB0, RB1, ou RB2
;et tout les 10 allumages de la Led le Buzzer doit vibrer un coup pendant 20msec ;
;
;Pour le mode programme Lent le mtronome va battre de la mme faon (led et buzzer ) pendant
;50 coups tout les X secondes puis 100 coups  une cadence plus leve puis  nouveau  une
cadence encore plus rapide jusqu a arret du mtronome (interrupteur).
;Le mode Rapide n'aura que des temps diffrents du mode lent.
;
#DEFINE	Led_Rouge	PORTA,0
#DEFINE 	Buzzez  	PORTA,1
#define 	Inter1	PORTB,0

#define 	Inter2	PORTB,1
#define 	Inter3	PORTB,2
#define 	Mode		PORTB,4		; 0=Lent 1=Rapide




CptSec          EQU 	RAMstart
Cpt1            EQU 	RAMstart+1     ; A counter, used multiple places
count1          EQU	RAMstart+2
count2          EQU	RAMstart+3
count3          EQU	RAMstart+4
LCDtmp          EQU	RAMstart+5
LCDtmp1         EQU	RAMstart+6
ASCII_O         EQU     RAMstart+7      ; ASCII One's digit to print
ASCII_T         EQU     RAMstart+8      ; ASCII Ten's digit to print
ASCII_H         EQU     RAMstart+9      ; ASCII Hundred's digit to print
; variable pour convertion binaire --> BCD 16bits
bdixmil         EQU 	RAMstart+10
bmille          EQU 	RAMstart+11
bcent		EQU 	RAMstart+12
bdix		EQU 	RAMstart+13
bun		EQU 	RAMstart+14
btmp		EQU 	RAMstart+15

valx		EQU 	RAMstart+16
val1		EQU 	RAMstart+18
val2		EQu	Ramstart+20
Nt1		EQU	RamStart+22    ; Nb de tours de pedale
Nt2		EQU	RamStart+24    ; Nb de tour de pedale pour Hectometre
Nt3		EQU	RamStart+26    ; Nb de tour de pedale pour Kms
Kms		EQU	RamStart+28
safe_w	EQU	RamStart+30		;not really temp, used by interrupt svc
safe_s	EQU	RamStart+31
COMND		EQu	RamStart+32
Tempo_H	Equ RamStart+33
Tempo_L	Equ Ramstart+34
drapeau     EQU     RamStart+35
LastRam     EQU     RamStart+35

	IF ( (LastRam)> 0x2F  )		; > RamStart+35
		ERROR   "Attention debordement zone RAM"
	ENDIF
;**************************************************************
; Program start
;**************************************************************
	ORG     RESET_V                 ; RESET vector location
RESET           GOTO    DEBUT

;--------------------------
TABLE_ST
            ifdef miniSimDE
               ADDLW 0xFF      ;-1      car PCL increment sur instruction MOVWF !!!
            endif
            MOVWF  PCL

MSG_N      ;     nop 	          <- autre solution rajouter un nop
                RETLW   'N'
		RETLW   'T'
		RETLW   'P'
		RETLW   'D'
		RETLW   '='
		RETLW   0x00

MSG_S           RETLW   'S'
		RETLW   'E'
		RETLW   'C'
		RETLW   'O'
		RETLW   'N'
                RETLW   'D'
                RETLW   'E'
		RETLW   '='
		RETLW   0x00

;--------
; INTERRUPT SERVICE ROUTINE
;--------
		org	0x04		;interrupt vector
Intsvc	movwf	safe_w		;save w
		swapf	STATUS,w	;swap status, w
		movwf	safe_s		;save status(nibble swap, remember)

; increment period count
		incf	Tempo_L,f
		btfsc	STATUS,Z
		incf	Tempo_H,f
		btfsc	STATUS,Z
		goto	cpt_over
;--------
; debordement du compteur FFFF -> 0 donc 1 seconde ecoulee
; re-init compteur( de 64 interrupts)
;--------
		;0 - 15625 = 49910 => 0xC2F6
		;movlw	0xC2
                movlw 0x20
		movwf	Tempo_H
		movlw	0xFF
		movwf	Tempo_L
		bsf	Drapeau,2		;notify Keep_time
		incf CptSec
	       incf	Nt1+1,F         ; incremente LSB
		BTFSC	STATUS,Z        ; si debordement MSB
		incf	Nt1,F
		incf	Nt2+1,F	        ; incremente LSB
		BTFSC	STATUS,Z	; si debordement MSB
		incf	Nt2,F
		incf	Nt3+1,F	        ; incremente LSB
		BTFSC	STATUS,Z	; si debordement MSB
		incf	Nt3,F




cpt_over
;--------
; done working, start restoring
;--------
		swapf	safe_s,w	;fetch status, reswap nibbles
		movwf	STATUS		;restore status
		swapf	safe_w,f	;swap nibbles in preparation
		swapf	safe_w,w	;for the swap restoration of w
		bcf	INTCON,2	;clear interrupt flag before return
		retfie			;return from interrupt


;*****************************************************************************
; Initialize processor registers
;*****************************************************************************
DEBUT                                   ; POWER_ON Reset (Beginning of program)
		CLRF    STATUS          ; Do initialization, Select bank 0
		CLRF    INTCON          ; Clear int-flags, Disable interrupts
		CLRF    PCLATH          ; Keep in lower 2KByte
		CLRF    PORTA           ; ALL PORT outputs should output Low.
		CLRF    PORTB
; ------------
; Effacer RAM
; ------------
	movlw	0x0c		; initialisation pointeur
	movwf	FSR		; pointeur d'adressage indirect
init1
	clrf	INDF		; effacer ram
	incf	FSR,f		; pointer sur suivant
	btfss	FSR,6		; tester si fin zone atteinte (=64)
	goto	init1		; non, boucler
	btfss	FSR,4		; tester si fin zone atteinte (=16) (64+16=80)
	goto	init1		; non, boucler
;---------------------
; initialisation registres
;-----------------------
		bank1
        	MOVLW   0x00           ; RA4 RA3-2-1-0 outputs
		MOVWF   TRISA
		MOVLW   0x0F           ; RB7-6-5-4 outputs RB 3-2-1-0 inputs
		MOVWF   TRISB
		MOVLW	B'00000111'	; Option register setup:
					; RPBU = 0 RB Pullups enabled
					; INTEDG = X
					; TOCS = 0  TMR0 fed from internal clock
					; TOSE = X
					; PSA = 0  Assign prescaler to TMR0
					; PS2-PS0= 101  = 64  prescaling ratio (111=256)
		MOVWF	OPTION_REG
		bank0
		clrf	TMR0			;start timer
		BSF INTCON, TOIE ;5	; enable TOIE interrupt TMR0
		BSF INTCON, GIE	      ; enable all interrupt

		CALL    LCD_Init        ; Initialize LCDisplay

;============================
Pres
     	call LCD_Cls
	clrf Cpt1
	MOVLW	LINE1	;
	CALL	LCD_Cde		; Set to first line
Pres1

      movf Cpt1,w
	CALL read_EE
	incf Cpt1,f
	ANDLW	0x0FF		; test sur (W)
	BTFSC	STATUS, Z	; zero= fin de message ?
	GOTO	Pres2
	Call LCD_Putchar
	goto Pres1
Pres2
        call delay_2_5s
        call LCD_Clear1

Boucle_ICI

; si evolution compteur par interrupt
 ;               call LCD_Clear1
 ;              MOVF CptSec,W
 ;               call bcd3
 ;          goto  Boucle_ICI
trait1
	; incremente les 3 compteur s
	;	incf	Nt1+1,F         ; incremente LSB
	;	BTFSC	STATUS,Z        ; si debordement MSB
	;	incf	Nt1,F
	;	incf	Nt2+1,F	        ; incremente LSB
	;	BTFSC	STATUS,Z	; si debordement MSB
	;	incf	Nt2,F
	;	incf	Nt3+1,F	        ; incremente LSB
	;	BTFSC	STATUS,Z	; si debordement MSB
	;	incf	Nt3,F


         	call LCD_Clear1
		MOVLW   LINE1
		CALL    LCD_Cde
		MOVLW   MSG_S
		CALL    TABLE_MSG
	       call LCD_Clear2
		movf 	CptSec,W
		call bcd3
goto  Boucle_ICI

	;comptage Hectometre
        	movf 	Nt2,W
		movwf	val1
		movf 	Nt2+1,W
		movwf	val1+1
		movlw  Tpd1/256
		movwf val2
		movlw (Tpd1-(Tpd1/256))
		movwf val2+1
		call comp_signed16     ; 43 tdp ?
		btfss STATUS,Z	       ; Z=1 si mesure=consigne
		goto	Suite1
	goto Boucle_ICI


Suite1

		movf 	Nt3,W
		movwf	val1
		movf 	Nt3+1,W
		movwf	val1+1
		movlw  Tpd2/256		;Tp2=434 tours pour 1km
		movwf val2
		movlw (Tpd2-(Tpd2/256))
		movwf val2+1
		call comp_signed16    ;mesure<consigne?
		btfss STATUS,Z
		goto Suite2

		clrf Nt3
		clrf Nt3+1
Suite2
	; affiche Nb de Tour de pedale

              	call LCD_Clear2
		MOVLW   LINE2
		CALL    LCD_Cde         ; Set cursor leftmost on line 2
		MOVLW   MSG_N           ; Point to Command message
		CALL    TABLE_MSG       ; Display the message
		MOVLW   LINE2+8
		CALL    LCD_Cde
		movf 	Nt1,W
		movwf	val1
		movf 	Nt1+1,W
		movwf	val1+1
		call bcd5V


		goto Boucle_ICI

; Fin du programme principal
;======================================
comp_signed16:
	movf val1,w
	xorlw 0x80
	movwf valx
	movf val2,w
	xorlw 0x80
	subwf valx,w	; subtract val1-val2
	goto Are_they_equal
comp_unsigned_16:
	movf val2,w
	subwf val1,w ; subtract Y-X
Are_they_equal:
	; Are they equal ?
	skpz
	goto result_comp
	; yes, they are equal -- compare lo
	movf val2+1,w
	subwf val1+1,w	; subtract Y-X
result_comp:
	; if val2=val1 then now Z=1.
	; if val1<val2 then now C=0.
	; if val2<=val1 then now C=1.
	return

;**************************************************************
; Envoi de message via table de message dans la zone programme
;**************************************************************
TABLE_MSG
		MOVWF   Cpt1         ; Point to the first char. we want
LOOP_WR
		MOVF   Cpt1,W         ; Renew the pointer
		CALL    TABLE_ST        ; Initiate table lookup
		XORLW   0x00            ; Is this the terminating char?
		BTFSC   STATUS,Z
		GOTO    END_WR
		CALL    LCD_Putchar      ; Actually put the char on display
		INCF    Cpt1,f	; Point to the next char
		GOTO    LOOP_WR         ; Do next character
END_WR
		RETURN
;=========================
; rev 26 janv 2003 lcd4c.inc
; fichiers lis:   delais_b.inc  lcd4c.h
;	routines en mode 4 Bits avec afficheur 2*16  base de HD77480
;	LCD_Init : initialisation de l'afficheur
;	LCD_Putchar : envoi un caractere ( contenu dans w ) sur l'afficheur
;	LCD_Cde  : envoi une commande ( contenue dans w)  l'afficheur
;	LCD_Cls  : efface l'afficheur
;	LCD_Clear1 : efface la premiere ligne de l'afficheur
;	LCD_Clear2 : efface la deuxieme ligne de l'afficheur
;	LCD_Home : renvoi le curseur en haut a gauche
;	LCD_Line1 : renvoi le curseur au debut de la ligne 1
;	LCD_Line2 : renvoi le curseur au debut de la ligne 2
;	pour positionner le curseur sur la ligne 1 :	movlw 0x80+offset	call LCD_Cde
;	pour positionner le curseur sur la ligne 2 : 	movlw 0xC0+offset	call LCD_Cde
;
;	variables a definir dans le programme principal:
;	LCDtmp,LCDTmp1			; LCD
;
;	constantes a definir dans le programme principal:
;	LCD_DATA	EQU PORTB	; LCD: Port ou est branche le bus de donnee ( bits 4  7 )
;	LCD_DATA_TRIS	EQU TRISB	; LCD: Port ou est branche le bus de donnee ( bits 4  7 )
;	LCD_CTRL	EQU PORTA	; LCD: Port ou est branche le bus de commande
;	LCD_CTRL_TRIS	EQU TRISA	; LCD: Port ou est branche le bus de commande
;	LCD_E		EQU 1		; LCD: Ligne de commande de controle de l'afficheur
;not used LCD_RW		EQU 1		; LCD: Ligne de Lecture/Ecriture de l'afficheur
;	LCD_RS		EQU 0		; LCD: Ligne de selection de l'afficheur
;
;	IMPORTANT : ce programme est optimis --> ne pas changer l'ordre des commandes


LCD_Cls
	movlw	D'01'			; effacage de l'ecran
	call	LCD_Cde
	goto	delay_2_5ms

;-----------------
LCD_Clear1				; effacage de la 1ere ligne
	movlw	D'16'
	movwf	LCDtmp1
	movlw	LINE1			; 0x80
	call	LCD_Cde
	call	delay_2_5ms
LCDc111
	movlw	' '
	call	LCD_Putchar
	decfsz	LCDtmp1,f
	goto	LCDc111
LCD_Line1
	movlw	LINE1
	call	LCD_Cde
	goto	delay_2_5ms
;--------------

LCD_Clear2				; effacage de la deuxieme ligne
	movlw	D'16'
	movwf	LCDtmp1
	movlw	LINE2			; 0xC0
	call	LCD_Cde
	call	delay_2_5ms
LCDcl21
	movlw	' '
	call	LCD_Putchar
	decfsz LCDtmp1,f
	goto	LCDcl21
LCD_Line2
	movlw	LINE2
	call	LCD_Cde
	goto	delay_2_5ms

LCD_Home
	movlw	D'02'
	call	LCD_Cde
	goto delay_2_5ms

LCD_Putchar
	bank0				; rajout 7/03/03
	movwf	LCDtmp
	call	delay_1ms
	movlw   0x0f
	andwf   LCD_DATA,F
	movf    LCDtmp,w
	andlw   0xF0
	iorwf   LCD_DATA,F
	ifdef miniSimDE
	bcf     LCD_CTRL, LCD_RW
	nop
	endif
	bsf     LCD_CTRL, LCD_RS
	nop
	bsf     LCD_CTRL, LCD_E
	call		delay_25us
	bcf     LCD_CTRL, LCD_E
	call		delay_25us
	movlw   0x0f
	andwf   LCD_DATA,F
	swapf   LCDtmp,W
	andlw   0xF0
	iorwf   LCD_DATA,F
	nop
	bsf     LCD_CTRL, LCD_E
	call		delay_25us
	bcf     LCD_CTRL, LCD_E
	call		delay_25us
	return

LCD_Init
	bcf     LCD_CTRL, LCD_E
	nop
	bcf     LCD_CTRL, LCD_RS
	nop
	ifdef miniSimDE
	bcf     LCD_CTRL, LCD_RW
	endif
	call 	delay_2_5ms
	movlw	0x0f
	andwf	LCD_DATA,F
	movlw	0x030			; Commande pour interface 4 bits
	iorwf	LCD_DATA,F
	bank1
	movlw   0x0F
	andwf   LCD_DATA_TRIS,W
	movwf   LCD_DATA_TRIS		; Port en sortie
	bank0
	bsf     LCD_CTRL, LCD_E
	nop
	bcf     LCD_CTRL, LCD_E
	call 	delay_2_5ms
	bsf     LCD_CTRL, LCD_E
	nop
	bcf     LCD_CTRL, LCD_E
	call 	delay_2_5ms
	bsf     LCD_CTRL, LCD_E
	nop
	bcf     LCD_CTRL, LCD_E
	call 	delay_100ms		; tempo 100ms , remplace l'attente Wait_Busy de l'afficheur
	movlw   0x0f
	andwf   LCD_DATA,F
	movlw   0x020           ; Commande pour interface 4 bits
	iorwf   LCD_DATA,F
	bsf     LCD_CTRL, LCD_E
	nop
	bcf     LCD_CTRL, LCD_E
	movlw   0x028		; Envoi de la commande Fonction 4-bit , Font, Nombre de lignes
	call    LCD_Cde
	movlw   0x008		; display off
	call    LCD_Cde
	movlw   0x00C		; display on (0x00C sans curseur, 0x00E avec , 0x00F clign. )
	call    LCD_Cde
	movlw   0x006

LCD_Cde
	movwf   LCDtmp
	ifdef miniSimDE
	call 	delay_100ms		; tempo 100ms , remplace l'attente Wait_Busy de l'afficheur
	endif
	movlw   0x0f
	andwf   LCD_DATA,F
	movf    LCDtmp,w
	andlw   0xF0
	iorwf   LCD_DATA,F
	ifdef miniSimDE
	bcf     LCD_CTRL,LCD_RW
	nop
	endif
	bcf     LCD_CTRL,LCD_RS
	nop
	bsf     LCD_CTRL,LCD_E
	nop
	bcf     LCD_CTRL,LCD_E
	movlw   0x0f
	andwf   LCD_DATA,F
	swapf   LCDtmp,W
	andlw   0xF0
	iorwf   LCD_DATA,F
	bsf     LCD_CTRL,LCD_E
	nop
	bcf     LCD_CTRL,LCD_E
	return
;-----------------------
;delais4M.inc pour Fquartz=4Mhz
;last update: 28 mars 2003
;count1 count2 count3  en bank0 pour delay routines
;delay_RS special pour routine d'attente bit 4800 Bauds
;delay : 5(count3-1)+1282(count2-1)+327684(count1-1)+6
;delay_25us delay_100us  delay_200us
;delay_1ms delay2_5ms  delay_10ms delay_100ms delay_500ms
;delay_1sec delay_10sec
;---------------------
;special pour routine d'attente bit 4800 Bauds
delay_RS
	   MOVLW           67           ; .67 pour 4800 bauds
         MOVWF           count1
         DECFSZ          count1,F
         GOTO            $-1
         Return
;------------------------
delay           movlw   0x001
                movwf   count1
                movlw   0x001
                movwf   count2
                movlw   0x020
                movwf   count3
delay1          decfsz  count3,f
                goto    $-1
                decfsz  count2,f
                goto    $-3
                decfsz  count1,f
                goto    $-5
        return
;--------------

delay_25us      movlw   0x01
                movwf   count1
                movlw   0x01
                movwf   count2
                movlw   0x05
                movwf   count3
                goto    delay1

delay_100us     movlw   0x01
                movwf   count1
                movlw   0x01
                movwf   count2
                movlw   20
                movwf   count3
                goto    delay1

delay_200us     movlw   0x01
                movwf   count1
                movlw   0x01
                movwf   count2
                movlw   40
                movwf   count3
                goto    delay1

delay_1ms       movlw   0x01
                movwf   count1
                movlw   0x01
                movwf   count2
                movlw   200
                movwf   count3
                goto    delay1

delay_2_5ms       movlw   0x01
                movwf   count1
                movlw   0x02
                movwf   count2
                movlw   243
                movwf   count3
                goto    delay1


delay_10ms       movlw   0x01
                movwf   count1
                movlw   8
                movwf   count2
                movlw   205
                movwf   count3
                goto    delay1

delay_100ms     movlw   0x01
                movwf   count1
                movlw   79
                movwf   count2
                movlw   0x01
                movwf   count3
                goto    delay1

delay_500ms	movlw 2
		movwf count1
		movlw 135
		movwf count2
		movlw 106
		movwf count3
		goto delay1

delay_1s      	movlw   4
                movwf   count1
                movlw   14
                movwf   count2
                movlw   56
                movwf   count3
                goto   delay1

delay_2_5s      movlw   8
                movwf   count1
                movlw   161
                movwf   count2
                movlw   218
                movwf   count3
                goto   delay1

delay_10s     	movlw   31
                movwf   count1
                movlw   133
                movwf   count2
                movlw   51
                movwf   count3
                goto   delay1
;-----------------------

; variable pour convertion binaire --> BCD 16bits
;bcent	EQU 	RAMstart+32
;bdix		EQU 	RAMstart+33
;bun		EQU 	RAMstart+34
;btmp		EQU 	RAMstart+35
;btmp1	EQU 	RAMstart+36
;val1		EQU 	RAMstart+37		; variable pour stockage valeur 16 bits
;val2		EQU 	RAMstart+39
;===============================================

bcd3	movwf  btmp		; affichage octet --> decimal 3 digits ( de 0 a 255 )
	movlw  '0'
        movwf  bcent
	movwf  bdix
bcd31	movlw  100
        subwf  btmp,w
	bnc    bcd32
	movwf  btmp
	incf   bcent,f
	goto   bcd31
bcd32   movlw  10
	subwf  btmp,w
	bnc    bcd33
	movwf  btmp
	incf   bdix,f
	goto   bcd32
bcd33	movlw  '0'
	addwf  btmp,w
	movwf  bun
	movf   bcent,w
	call   LCD_Putchar
	movf   bdix,w
	call   LCD_Putchar
	movf   bun,w
	call   LCD_Putchar
	return


; affichage octet --> decimal 5 digits , de 0 a 65535
; avec blanc sur les chiffres non significatifs ....1   ...1
bcd5V   bCf drapeau,0
	movf	val1,w
	movwf	valx
	movf	val1+1,w
	movwf	valx+1

	swapf  val1,w
	iorlw  0xf0
	movwf  bmille
	addwf  bmille,f
	addlw  0xE2
	movwf  bcent
	addlw  0x32
	movwf  bun
	movf   val1,w
	andlw  0x0F
	addwf  bcent,f
	addwf  bcent,f
	addwf  bun,f
	addlw  0xE9
	movwf  bdix
	addwf  bdix,f
	addwf  bdix,f
	swapf  val1+1,w
	andlw  0x0F
	addwf  bdix,f
	addwf  bun,f
	rlf    bdix,f
	rlf    bun,f
	comf   bun,f
	rlf    bun,f
	movf   val1+1,w
	andlw  0x0F
	addwf  bun,f
	rlf    bmille,f
	movlw  0x07
	movwf  bdixmil
        movlw  0x0A
Lb1: 	addwf  bun,f
	decf   bdix,f
	btfss  STATUS,C
	goto   Lb1
Lb2:	addwf  bdix,f
	decf   bcent,f
	btfss  STATUS,C
	goto   Lb2
Lb3:	addwf  bcent,f
	decf   bmille,f
	btfss  STATUS,C
	goto   Lb3
Lb4:	addwf  bmille,f
        decf   bdixmil,f
	btfss  STATUS,C
        goto   Lb4

	movf   bdixmil,w
        BTFSC STATUS,Z
        BTFSC drapeau,0
        goto Lb40a
        movlw ' '
        goto Lb40b
Lb40a:  bsf drapeau,0
	addlw  '0'
Lb40b:  call   LCD_Putchar

	movf   bmille,w
        BTFSC STATUS,Z
        BTFSc drapeau,0
        goto Lb41a
        movlw ' '
        goto Lb41b
Lb41a:  bsf drapeau,0
	addlw  '0'
Lb41b:  call   LCD_Putchar

	movf   bcent,w
        BTFSC STATUS,Z
        BTFSc drapeau,0
        goto Lb42a
        movlw ' '
        goto Lb42b
Lb42a:  bsf drapeau,0
	addlw  '0'
Lb42b:  call   LCD_Putchar

	movf   bdix,w
        BTFSC STATUS,Z
        BTFSc drapeau,0
        goto Lb43a
        BTFSC drapeau,1         ; affichage ..0.0 kms
        goto Lb43a
        movlw ' '
        goto Lb43b
Lb43a:  bsf drapeau,0
	addlw  '0'
Lb43b:  call   LCD_Putchar

        BTFSS drapeau,1         ; affichage ...0.0 kms
        goto Lb44               ; si drapeau ......X. = 1
	movlw	'.'             ; sinon ....0
	call LCD_Putchar
Lb44:	movf   bun,w
	addlw  '0'
 	call   LCD_Putchar
	movlw	' '
	call LCD_Putchar
	movf valx,w
	movwf val1
	movf valx+1,w
	movwf	val1+1
        BCF drapeau,1
	return
;-------------------
; voir 84_IRC5_.h pour environnment et variables
;*************************************************

read_EE

	movwf   EEADR           ; defini offset addresse
	bank1			; 16F84 !!!!
	bsf     EECON1,RD       ; mode lecture
	bank0			; 16F84 !!!!
	movf    EEDATA,W       	; recupere lecture

	return


	ORG EEPROM
      ;"1234567890123456"
EE0 DT "miSimDE 1.7.7   ",0
EE1 DT "chrono4Z.asm    ",0
EE2 DT "10 fev 2005    ",0

		END





















