;-------------------------------------------------------------------------;
; SHOTMR0.ASM     Show TMR0 in decimal on LCD after counting 1 second     ;
;-------------------------------------------------------------------------;


        include <p16f84_.inc>
	ifdef miniSimDE
	p=16F84
	else
	ERRORLEVEL -302        ;  supress message because of page change
;        ERRORLEVEL -224        ;  supress annoying message because of tris
	Processor  16F84
	endif
	Radix           DEC
	EXPAND
;	__CONFIG   _CP_OFF & _WDT_OFF & _PWRTE_ON & _XT_OSC
        __CONFIG _PWRTE_ON & _LP_OSC & _WDT_OFF  
; uses 32.768 kHz crystal

;  Define Information
#DEFINE RS PORTA, 0          ;  RA0 is RS line of LCD
#DEFINE E  PORTA, 1          ;  RA1 is E line of LCD
                                  ;  RB0-RB3 are D4-D7 of LCD
;  Macro

EStrobe MACRO                   ;  Strobe the "E" Bit
        bsf    E
         bcf    E
        ENDM

#define RAMstart 0x0C			; pour 16F84

Temp            EQU 	RAMstart   ; a temporary variable               
count           EQU 	RAMstart+1 ; counter
precount        EQU 	RAMstart+2 ; holds value for prescalar               
bin             EQU 	RAMstart+3 ; binary number to be converted to BCD              
hundreds        EQU 	RAMstart+4 ; BCD hundreds result              
tens_and_ones   EQU 	RAMstart+5 ; BCD tens and ones result
          

            ORG 0x000               ; start at location 0

            goto main           ; jump over to main routine

             ORG 0x0004
                nop
                retfie
;-------------------------------------------------------------------------;
;                    Data for message to be output                        ;
;-------------------------------------------------------------------------;
shomsg:                         ;  Message to Output
  addwf  PCL, f                 ;  Output the Characters
m0  dt     "TMR0 Value:", 0
m1  dt     "Multiply by:", 0

;-------------------------------------------------------------------------;
;                        Ratios of Prescaler                              ;
;-------------------------------------------------------------------------;
preratio:
  addwf  PCL, f                 ;  select offset using W
 ;   dt     D'2',D'4',D'8',D'16',D'32',D'64',D'128'
    dt     2,4,8,16,32,64,128
;-------------------------------------------------------------------------;
;                       Initialize the ports                              ;
;-------------------------------------------------------------------------;
Init:
            clrf   PORTA
            clrf   PORTB
               bank1
            movlw B'00010000'          ; RA4 input, others outputs
            movwf PORTA
            movlw B'00110000'          ; RB4, RB5 input, others outputs
            movwf PORTB
            movlw B'00100001'          ; pull-ups enabled
                                       ; prescaler assigned to RA4
                                       ; prescaler set to 1:4
   
        bank0
     ;       option
            clrf precount
            return

;-------------------------------------------------------------------------;
;                 Initialize the LCD                                      ;
;-------------------------------------------------------------------------;
initlcd:
  movlw 40
  call   nmsec                  ;  Wait 40 msecs before Reset
  bcf    RS                     ;  send an 8 bit instruction
  movlw  0x03                   ;  Reset Command
  call   NybbleOut              ;  Send the Nybble
  call   Dlay5                  ;  Wait 5 msecs before Sending Again
  EStrobe
  nop
  nop                           ;  Wait 244 usecs before Sending the Second Time
  EStrobe
  nop
  nop                           ;  Wait 244 usecs before Sending the Third Time
  bcf    RS                     ; send an 8 bit instruction
  movlw  0x02                   ;  Set 4 Bit Mode
  call   NybbleOut
  nop
  nop
  movlw  0x028                  ;  4 bit, 2 Line, 5x7 font
  call   SendINS
  movlw  0x010                  ;  display shift off
  call   SendINS
  movlw  0x001                  ;  Clear the Display RAM
  call   SendINS
  call   Dlay5                  ;  Note, Can take up to 4.1 msecs
  movlw  0x006                  ;  increment cursor
  call   SendINS
  movlw  0x00C                  ;  display on cursor off
  call   SendINS
  return

;-------------------------------------------------------------------------;
;              Send the character in W out to the LCD                     ;
;-------------------------------------------------------------------------;
SendASCII
  addlw '0'                     ;  Send nbr as ASCII character
SendCHAR                        ;  Send the Character to the LCD
  movwf  Temp                   ;  Save the Temporary Value
  swapf  Temp, w                ;  Send the High Nybble
  bsf    RS                     ;  RS = 1
  call   NybbleOut
  movf   Temp, w                ;  Send the Low Nybble
  bsf    RS
  call   NybbleOut
  return

;-------------------------------------------------------------------------;
;              Send an instruction in W out to the LCD                    ;
;-------------------------------------------------------------------------;
SendINS                         ;  Send the Instruction to the LCD
  movwf  Temp                   ;  Save the Temporary Value
  swapf  Temp, w                ;  Send the High Nybble
  bcf    RS                     ;  RS = 0
  call   NybbleOut
  movf   Temp, w                ;  Send the Low Nybble
  bcf    RS
  call   NybbleOut
  return

;-------------------------------------------------------------------------;
;              Send the nibble in W out to the LCD                        ;
;-------------------------------------------------------------------------;
NybbleOut                       ;  Send a Nybble to the LCD
  movwf  PORTB
  EStrobe                       ;  Strobe out the LCD Data
  nop
  nop
  return

;-------------------------------------------------------------------------;
;                   Output the message on the LCD                           ;
;-------------------------------------------------------------------------;
OutMessage:
  movwf  FSR                    ;  Point at first letter
OutLoop:
  movf   FSR, w                 ;  Get pointer into W
  incf   FSR, f                 ;  Set up for next letter
  call   shomsg                 ;  Get character to output
  iorlw  0                      ;  At the End of the Message?
  btfsc  STATUS, Z              ;  Skip if not at end
  return                        ;  Yes - Equal to Zero
  call   SendCHAR               ;  Output the ASCII Character
  goto   OutLoop                ;  Get the next character

;-------------------------------------------------------------------------;
;                  Change binary nbr in bin to BCD                        ;
;-------------------------------------------------------------------------;
binary_to_bcd                     ; by Scott Dattalo
  clrf hundreds
  swapf bin, W
  addwf bin, W
  andlw B'00001111'
  skpndc
    addlw 0x16
  skpndc
    addlw 0x06
  addlw 0x06
  skpdc
  addlw -0x06
  btfsc bin,4
    addlw 0x16 - 1 + 0x6
  skpdc
    addlw -0x06
  btfsc bin,5
    addlw 0x30
  btfsc bin, 6
    addlw 0x60
  btfsc bin,7
    addlw 0x20
  addlw 0x60
  rlf hundreds, f
  btfss hundreds, 0
    addlw -0x60
  movwf tens_and_ones
  btfsc bin,7
    incf hundreds, f
  return

;-----------------------------------------------------------------------;
;                           Delay routine                               ;
;-----------------------------------------------------------------------;
msec250  movlw 0               ; 250 msec delay (adjusted to try and
                               ; allow for 2.5% low loop time)
         goto $+2
Dlay5    movlw 5               ; delay for 5 milliseconds
nmsec:                         ; delay for # msec in W on entry
         nop                   ; each nop is 0.122 milliseconds
         nop
         nop                   ; each total loop is 8 X 0.122 = 0.976 msec
         nop
         addlw H'FF'           ; same as subtracting 1 from W
         btfss STATUS, Z       ; skip if result is zero
         goto nmsec            ; this is  2 X 0.122 msec
         return                ; back to calling point

;-------------------------------------------------------------------------;
;                Display binary value in W in decimal                     ;             ;
;-------------------------------------------------------------------------;
DispDec
              movwf bin
              call binary_to_bcd
              movf hundreds, W
              call SendASCII
              swapf tens_and_ones, W
              andlw H'F'
              call SendASCII
              movf tens_and_ones, W
              andlw H'F'
              call SendASCII
              return

;-------------------------------------------------------------------------;
;             Return the prescalar ratio setting in W                     ;
;-------------------------------------------------------------------------;
getpre:
       bsf STATUS, RP0         ; change to page 1
       movf OPTION_REG, W      ; get OPTION register
       bcf STATUS, RP0         ; back to page 0
       andlw 7                 ; get 1st 3 bits in W
       return

;-------------------------------------------------------------------------;
;        Check pushbuttons and increment or decrement prescalar           ;
;-------------------------------------------------------------------------;
ckbuttons:
              call getpre           ; get prescalar ratio # in Temp
              movwf Temp
              btfss PORTB, 4        ; Skip if button on RB4 up
              goto preup            ; increment prescalar
              btfss PORTB, 5        ; Skip if button on RB5 up
              goto predown          ; decrement prescalar
              return                ; no buttons pressed, return
preup:
              incf Temp, f          ; up one
              btfsc Temp, 3         ; if reach 8, go back to zero
              clrf Temp
              goto newpre           ; and fix new ratio
predown:
              decf Temp, f          ; down one
              movlw H'FF'           ; check for underflow
              subwf Temp, W
              btfss STATUS, Z       ; skip if underflow
              goto newpre           ; and fix new ratio
              movlw 7               ; reset to 7
              movwf Temp
newpre
              bsf STATUS, RP0      ; change to page 1
              movlw B'11111000'    ; mask out low three bits
              andwf OPTION_REG, W  ; result in W
              bcf STATUS, RP0      ; back to page 0
              iorwf Temp, W        ; add in new prescalar ratio #
              option               ; update OPTION
              return

;-------------------------------------------------------------------------;
;                           The Main routine                              ;
;-------------------------------------------------------------------------;
main:   nop
        call Init               ; initialize ports, set up timer
       call initlcd            ; initialize the LCD
       movlw H'80'             ; position at 1st line column 0
       call SendINS
       movlw m0 -2             ; send 'TMR0 Value:' message
       call OutMessage
       movlw H'C0'
       call SendINS            ; position at 2nd line column 0
       movlw m1 -2             ; send 'Multiply by:' message
       call OutMessage
sholoop:
       call ckbuttons          ; update prescalar if buttons pushed
       movlw H'CD'             ; position at  2nd line column 13
       call SendINS
       call getpre             ; get prescalar ratio # in binary
       call preratio           ; change to prescalar ratio
       call DispDec            ; put it on LCD
       movlw H'8C'             ; position at 1st line column 12
       call SendINS
       bcf INTCON, T0IF        ; clear timer zero interrupt flag
       clrf TMR0               ; zero TMR0
       call msec250            ; wait a total of one second
       call msec250
       call msec250
       call msec250
       movf TMR0, W            ; retrieve timer zero
       btfsc INTCON, T0IF      ; check if timer overflowed
       goto overload           ; yes, display 'OVR'
       call DispDec            ; display TMR0 value
       goto sholoop            ; repeat forever
overload
       movlw 'O'               ; output 'OVR'
       call SendCHAR
       movlw 'V'
       call SendCHAR
       movlw 'R'
       call SendCHAR
       goto sholoop            ; and continue

       end
