; version PICEmulateur
;27 juillet 2006

	include <p16f84_.inc>
ifNdef miniSimDE     
#define miniSimDE
ENDIF

ifdef miniSimDE
	p=16F84
#define processor p16F84
else
	ERRORLEVEL      -302
	Processor       16F84
endif
	Radix           DEC
	EXPAND

	__CONFIG   _CP_OFF & _WDT_OFF & _PWRTE_ON & _XT_OSC

;========== Hardware ==============
; 16F84
; 1 RA2 E (LCD)         18 RA1 W (LCD)
; 2 RA3  (BP RAZ )      17 RA0 RS (LCD)
; 3 RA4 (BP IMPULSION)  16 OSC1 Quartz 4Mhz
; 4 MCLR RESET          15 OSC2 Quartz
; 5 VSS -               14 VDD +
; 6 RB0 LCD             13 RB7 LCD 
; 7 RB1 LCD             12 RB6 LCD 
; 8 RB2 LCD             11 RB5 LCD 
; 9 RB3 LCD             10 RB4 LCD 
;====================================
; See AN587 for 4-bit and 8-bit interfaces LCD.
;***********************
; Fosc		= 4MHz
; Cycle_time	= 1/Fosc / 4 = 1/(4*10^6) / 4	= 1uSec
;***********************

;***********************
; Equates, I/O, vars
;***********************
RESET_V	EQU	0x0000		; Address of RESET Vector
ISR_V		EQU	0x0004		; Address of Interrupt Vector
OSC_FREQ	EQU	D'4000000'	; Oscillator Frequency is 4 MHz
LCD_DATA	EQU	PORTB		; LCD data lines interface
LCD_DATA_TRIS	EQU	TRISB
LCD_CTRL	EQU	PORTA		; LCD control lines interface
LCD_LINE0	EQU	0x000
LCD_LINE1	EQU	0x040
LCD_LINE2	EQU	0x014             
LCD_LINE3	EQU	0x054

; PORTA bits
	ifdef miniSimDE
LCD_E           EQU 2		; LCD: Ligne de commande de controle de l'afficheur
LCD_RW          EQU 1		; LCD: Ligne de Lecture/Ecriture de l'afficheur
LCD_RS          EQU 0		; LCD: Ligne de selection de l'afficheur-> RA0
	else
LCD_E           EQU 2		; LCD: Ligne de commande de controle de l'afficheur
LCD_RS          EQU 0		; LCD: Ligne de selection de l'afficheur-> RA0
	endif
; PORTB bits
DB7		EQU	7		; LCD dataline 7 (MSB)
DB6		EQU	6		; LCD dataline 6
DB5		EQU	5		; LCD dataline 5
DB4		EQU	4		; LCD dataline 4
DB3		EQU	3		; LCD dataline 3
DB2		EQU	2		; LCD dataline 2
DB1		EQU	1		; LCD dataline 1
DB0		EQU	0		; LCD dataline 0 (LSB)

                ORG 0x020               ; on pourrait commnencer  0x0C mais choix 0x20 pour compatible 16F628
LCD_TEMP	EQU	$		; LCD subroutines internal use
T1_INDEX	EQU	$+1		; Index to table1 strings
RsByte	        EQU	$+2		; Index to table2 strings
sequence	EQU	$+3		; Index to table3 strings
COUNT		EQU	$+4		; counter
DELAY		equ	$+5		; Used in DELAYxxx routines
X_DELAY	        equ	$+6		; Used in X_DELAYxxx routines
Cpt1		equ	$+7
Cpt2		equ	$+8
Cpt3		equ	$+9
Cpt4		equ	$+10
;---------------------
	ORG 0x20+12
	dummy		EQU $
	bcent		EQU $+1
	bdix		EQU $+2
	bun		EQU $+3
	btmp		EQU $+4
	btmp1		EQU $+5
	val1		EQU $+6	;2
	val2		EQU $+8	;2
	dixmil	        EQU $+10
	mille		EQU $+11
	cent		EQU $+12
	dix		EQU $+13
	un		EQU $+14

;EEPROM	EQU	0x2100
#define PULSE  PORTA,4        ; BP pulse sur RA4 
#define RAZ    PORTA,3        ; BP raz sur RA1
;***********************
; Debut du programme
;***********************
	ORG	RESET_V		
RESET		GOTO	START


;***********************
; Traitement interruption               ;interrupt  non utilis dans ce programme!
;***********************
	ORG	ISR_V			; Interrupt vector location
INTERRUPT	BCF     STATUS, RP0	; Select bank 0
		GOTO    INTERRUPT

;***********************
; Initialisation
;***********************
START					; POWER_ON Reset (Beginning of program)
		CLRF	STATUS		; Do initialization, Select bank 0
		CLRF	INTCON		; Clear int-flags, Disable interrupts
		CLRF	PCLATH		; Keep in lower 2KByte

		MOVLW 0xFF
                movwf PORTA		; ALL PORT output should output Low.
		CLRF	PORTB

		BSF	STATUS, RP0	; Select bank 1
		MOVLW	b'00011000'	; RA2,RA1,RA0 outputs, RA4-RA3 inputs
		MOVWF	TRISA
		MOVLW	0x000		; RB7-0 outputs
		MOVWF	TRISB
		BCF	OPTION_REG, NOT_RBPU
					; Disable PORTB pull-ups
		BCF	STATUS, RP0	; Select bank 0
 		CLRF	INTCON		; Clear int-flags, Disable interrupts
  
                clrf Cpt1
               
		CALL	LCDINIT		; Initialize LCDisplay
		MOVLW	LCD_LINE0
		CALL	LCDSDDA		; Position cursor leftmost on first line
		movlw   EE1-EE0           ; ajuste pointeur sur 2em message en EEPROM
              	MOVWF  Cpt4
	        call   Msg_Eeprom         ; affiche message en eeprom
                call Tempo_1S
 ;============== boucle principale =============
Boucle:
		MOVLW	LCD_LINE1
		CALL	LCDSDDA		; Position cursor leftmost on first line
		CALL	T1_MSG		; Display message
 	        MOVLW	LCD_LINE1 + 12
		CALL	LCDSDDA
		movf Cpt1,w	        ; ecrit la valeur du compteur en BCD sur l'afficheur (000  255)
		call Bcd3
    
L1
                btfsC RAZ               ; 0 = RAZ    etat 1 si non activ  (R de rappel au +5V)
                goto L2
                call Raz_Compteur
                goto Boucle

;***Test changement sur entree
L2              btfss PULSE            ; comptage sur etat zero  etat 1 au repos= non activ
                goto L3
                goto L1
L3      ; attente relachement BP
                call DELAY500           ; delay antirebond de 500S (@4MHz)
L4              btfsC PULSE
                goto L5
                goto L2
L5        
                incf Cpt1,f
        
         	call DELAY500
		goto boucle

;=======================================================================

  Raz_Compteur
                clrf Cpt1
                call LCDCLEAR 
		MOVLW	LCD_LINE0  
		CALL	LCDSDDA		; Position cursor leftmost on first line
 		movlw   EE2-EE0           ; ajuste pointeur sur 2em message en EEPROM
              	MOVWF  Cpt4
	        call   Msg_Eeprom         ; affiche message en eeprom
                call Tempo_1S
              	MOVLW	LCD_LINE0  
		CALL	LCDSDDA		; Position cursor leftmost on first line
 	        call Blankline
          	MOVLW	LCD_LINE1  
		CALL	LCDSDDA		; Position cursor leftmost on first line
 	        call Blankline           
                 return  

;=======================================
; Table message to display
;=====================================
MSG1
		addwf	PCL ,F		;Jump to char pointed to in W reg
		DT " Comptage="
                retlw	0
MSG1_END
		
	IF ( (MSG1 & 0x0FF) >= (MSG1_END & 0x0FF) )
		MESSG   "Warning - User Definded: Table 'MSG1' crosses page boundry in computed jump"
	ENDIF


;***********************
; Send a message using a table to output a message
;***********************
T1_MSG
		MOVLW	0		; index debut de table
T1_1
		MOVWF	T1_INDEX	; adresse message
		CALL	MSG1
		ANDLW	0x0FF		; Test fin de message 
		BTFSC	STATUS, Z	; par presence de zero  la fin
		GOTO	T1_END             
		CALL	LCDPUTCHAR	; affiche le caractere
		MOVF	T1_INDEX, W	; pointeur sur le suivant
		ADDLW	1
		GOTO	T1_1
T1_END
		RETURN
;*************************
BlankLine                               ; affiche une ligne blanche
		movlw 16
		movwf	Cpt3
Blk1:		movlw	' '
		CALL	LCDPUTCHAR		
		decfsz Cpt3,f
		goto Blk1
		return
;***********************
; LCD Module Subroutines
;***********************
LCDINIT
; Initilize LC-Display Module	 Busy-flag is not yet valid
		BCF LCD_CTRL,LCD_RW
		BCF LCD_CTRL,LCD_RS
		BCF LCD_CTRL,LCD_E
	; ALL PORT output should output Low.
		MOVLW	0x01E		; power-up delay
		CALL	X_DELAY500	; 30 * 0.5mS = 15mS
					; Busy Flag should be valid from here
		MOVLW	0x038		; 8-bit-interface, 2-lines
		CALL	LCDPUTCMD
		MOVLW	0x000		; disp.off, curs.off, no-blink
		CALL	LCDDMODE
		CALL	LCDCLEAR
		MOVLW	0x004		; disp.on, curs.off
		CALL	LCDDMODE
		MOVLW	0x002		; auto-inc (shift-cursor)
		CALL	LCDEMODE
		RETURN
;=========================
LCD_ENABLE
; Pulses LCD enable pin
		BSF	LCD_CTRL, LCD_E	; LCD E-line High
		BCF	LCD_CTRL, LCD_E	; LCD E-line Low
		RETURN
;=========================
LCDBUSY
; Returns when LCD busy-flag is inactive
		BSF	STATUS,RP0	; Select Register page 1
		MOVLW	0x0FF		; Set PORTB for input
		MOVWF	LCD_DATA_TRIS
		BCF	STATUS, RP0	; Select Register page 0
		BCF	LCD_CTRL, LCD_RS; Set LCD for command mode
		BSF	LCD_CTRL, LCD_RW; Setup to read busy flag
		BSF	LCD_CTRL, LCD_E	; LCD E-line High
		MOVF	LCD_DATA, W	; Read busy flag + DDram address
		BCF	LCD_CTRL, LCD_E	; LCD E-line Low
		ANDLW	0x80		; Check Busy flag, High = Busy
		BTFSS	STATUS, Z
		GOTO	LCDBUSY
LCDNOTBUSY	BCF	LCD_CTRL, LCD_RW
		BSF	STATUS, RP0	; Select Register page 1
		MOVLW	0x000
		MOVWF	LCD_DATA_TRIS	; Set PORTB for output
		BCF	STATUS, RP0	; Select Register page 0
		RETURN
;=========================
LCDCLEAR
; Clears display and returns cursor to home position (upper-left corner).
		MOVLW	0x001
		CALL	LCDPUTCMD
		RETURN
;=========================
LCDHOME
; Returns cursor to home position.
; Returns display to original position (when shifted).
		MOVLW	0x002
		CALL	LCDPUTCMD
		RETURN
;=========================
LCDEMODE
; Sets entry mode of display.
; Required entry mode must be set in W
;  b0	: 0 = no display shift	1 = display shift
;  b1	: 0 = auto-decrement	1 = auto-increment
;  b2-7	: don't care
		ANDLW	0x003		; Strip upper bits
		IORLW	0x004		; Function set
		CALL	LCDPUTCMD
		RETURN
;=========================
LCDDMODE
; Sets display control.
; Required display mode must be set in W
;  b0	: 0 = cursor blink off	1 = cursor blink on
;  b1	: 0 = cursor off	1 = cursor on
;  b2	: 0 = display off	1 = display on (display data remains in DDRAM)
;  b3-7	: don't care
		ANDLW	0x007		; Strip upper bits
		IORLW	0x008		; Function set
		CALL	LCDPUTCMD
		RETURN
;=========================
LCDSCGA
; Sets Character-Generator-RAM address. CGRAM is read/written after this setting.
; Required CGRAM address must be set in W
;  b0-5	: required CGRAM address  b6-7	: don't care

		ANDLW	0x03F		; Strip upper bits
		IORLW	0x040		; Function set
		CALL	LCDPUTCMD
		RETURN
;=========================
LCDSDDA
; Sets the Display-Data-RAM address. DDRAM data is read/written after
;  this setting.
; Required DDRAM address must be set in W
;  b0-6	: required DDRAM address,  b7	: don't care
		IORLW	0x080		; Function set
		CALL	LCDPUTCMD
		RETURN
;=========================
LCDGADDR
; Returns address counter contents, used for both DDRAM and CGRAM.
; RAM address is returned in W
		BSF	STATUS,RP0	; Select Register page 1
		MOVLW	0x0FF		; Set PORTB for input
		MOVWF	LCD_DATA_TRIS
		BCF	STATUS, RP0	; Select Register page 0
		BCF	LCD_CTRL, LCD_RS; Set LCD for command mode
		BSF	LCD_CTRL, LCD_RW; Setup to read busy flag
		BSF	LCD_CTRL, LCD_E	; LCD E-line High
		MOVF	LCD_DATA, W	; Read busy flag + RAM address
		BCF	LCD_CTRL, LCD_E	; LCD E-line Low
		ANDLW	0x07F		; Strip upper bit
		BCF	LCD_CTRL, LCD_RW
		BSF	STATUS, RP0	; Select Register page 1
		MOVLW	0x000
		MOVWF	LCD_DATA_TRIS	; Set PORTB for output
		BCF	STATUS, RP0	; Select Register page 0
		RETURN
;======================================
LCDPUTCHAR
; Sends character to LCD, Required character must be in W
		MOVWF	LCD_TEMP	; Character to be sent is in W
		CALL	LCDBUSY		; Wait for LCD to be ready
		BCF	LCD_CTRL, LCD_RW; Set LCD in read mode
		BSF	LCD_CTRL, LCD_RS; Set LCD in data mode
		BSF	LCD_CTRL, LCD_E	; LCD E-line High
		MOVF	LCD_TEMP, W
		MOVWF	LCD_DATA	; Send data to LCD
		BCF	LCD_CTRL, LCD_E	; LCD E-line Low
		RETURN
;=====================================
LCDPUTCMD
; Sends command to LCD Required command must be in W
		MOVWF	LCD_TEMP	; Command to be sent is in W
		CALL	LCDBUSY		; Wait for LCD to be ready
		BCF	LCD_CTRL, LCD_RW; Set LCD in read mode
		BCF	LCD_CTRL, LCD_RS; Set LCD in command mode
		BSF	LCD_CTRL, LCD_E	; LCD E-line High
		MOVF	LCD_TEMP, W
		MOVWF	LCD_DATA	; Send data to LCD
		BCF	LCD_CTRL, LCD_E	; LCD E-line Low
		RETURN
;=================================================

bcd3	movwf  btmp		; affichage octet --> decimal 3 digits ( de 0 a 255 )
	movlw  '0'
        movwf  bcent
	movwf  bdix
bcd31	movlw  100
        subwf  btmp,w
	bnc    bcd32
	movwf  btmp
	incf   bcent,f
	goto   bcd31
bcd32   movlw  10
	subwf  btmp,w
	bnc    bcd33
	movwf  btmp
	incf   bdix,f
	goto   bcd32
bcd33	movlw  '0'
	addwf  btmp,w
	movwf  bun
	movf   bcent,w
	call   LCDPUTCHAR
	movf   bdix,w
	call   LCDPUTCHAR
	movf   bun,w
	goto   LCDPUTCHAR

;----------------------
	
;=====================================
; Delay_time	= ((DELAY_value * 3) + 4) * Cycle_time
; DELAY_value	= (Delay_time - (4 * Cycle_time)) / (3 * Cycle_time)
; i.e. (@ 4MHz crystal)
; Delay_time	= ((32 * 3) + 4) * 1uSec = 100uSec
; DELAY_value	= (500uSec - 4) / 3 = 165.33	= 165
;-------------------
DELAY500
	MOVLW	D'165'		; +1		1 cycle
	MOVWF	DELAY		; +2		1 cycle
DELAY500_LOOP
	DECFSZ	DELAY, F	; step 1	1 cycle
	GOTO	DELAY500_LOOP	; step 2	2 cycles
	RETURN			; +3		2 cycles
;
X_DELAY500	
	MOVWF	X_DELAY		; +1		1 cycle
X_DELAY500_LOOP
	CALL	DELAY500	; step1		wait 500uSec
	DECFSZ	X_DELAY, F	; step2		1 cycle
	GOTO	X_DELAY500_LOOP	; step3		2 cycles
	RETURN			; +2		2 cycles
 
;--------------------
Tempo_1S			; tempo 1sec 
	movlw   10
	movwf   Dummy
ifdef miniSimDE                 ; pour accelerer en mode simulation
tp2     movlw 2
#define processor p16F84
else
tp2     movlw 200
endif
	call 	X_DELAY500
	decfsz  Dummy,f
	goto    tp2
	return

;----------------
read_EE	
	ifdef miniSimDE
	bank0	
	else
	bank1		; pour PIC16F628  only
	endif
	movwf   EEADR           ; defini offset addresse
	bank1
	bsf     EECON1,RD     ; mode lecture
	bank0
	movf    EEDATA,W       	; recupere lecture
	return
;-----------------------
Msg_Eeprom
	movf Cpt4,w
	ifdef miniSimDE
	bank0	
	else
	bank1		; pour PIC16F628  only
	endif
	MOVWF	EEADR
        bank1
	BSF 	EECON1,RD	;  bit 0
	bank0
	MOVF 	EEDATA,W
	incf	Cpt4,f
	ANDLW	0x0FF		
	BTFSC	STATUS, Z	
	Return
	call LCDPUTCHAR
	goto Msg_Eeprom
	return

;------------------------

org EEPROM
EE0 DT "PIC16F84 ",0
EE1 DT "PF 27juill 06",0
       ;1234567890123456
EE2 DT "RAZ comptage    ",0


	END				; End of program

