;Police: TERMINAL
;
;ͻ
;Recepteur Infra Rouge pour piloter le P.C. a distance    
;                                                         
;Ce montage est capable d'allumer un P.C. ATX a distance, 
;de choisir a distance une source audio parmis 3,         
;de choisir a distance une paire d'enceinte parmis 4,     
;de couper le son (mode mute),                            
;et bien sur de piloter n'importe quel logiciel pouvant   
;se commander en ligne de commande sous Linux.            
;ͼ
;
;La selection des sources, des enceintes, et l'allumage du P.C. a distance
;se font a partir du PORTB configure en sortie.
;Dans sa version de base (pas d'allumage a distance, pas de selection des sources et des enceintes),
;le montage n'a pas besoin d'alimentation externe.
;
;Le PIC a besoin de savoir a quelle vitesse emmet la telecommande. Pour cela,
;Vous devez mettre le PIC sous tension tout en maintenant enfoncees les deux touches.
;Appuyez alors un peu sur toutes les touches de votre telecommande jusqu'a ce que la LED
;ne clignote plus.
;Ensuite appuyez sur le premier bouton; et appuyer sur la touche de votre telecommande qui
;permettra d'allumer votre P.C.
;Procedez de meme pour la selection des sources, des enceintes, et la touche mute.
;Voila, le PIC est configure une fois pour toute (jusqu'a ce que vous changiez de telecommande).

;ͻ
;Enregistreur de code de telecommandes Infra Rouge
;ͼ
;Ne pas oublier que les bits transmis au ports serie le sont a partir du bit de poids le plus faible!!!

	list		p=16f84,f=inhx8m
	__config	B'11111111110001'
	#include	"p16f84a.inc"

preced	equ	H'0C'		;valeur precedente du PORTA
W_temp	equ	H'0D'		;Sauvegarde de W en cas d'interruption
ST_temp	equ	H'0E'		;Sauvegarde de Satus en cas d'interruption
index	equ	H'0F'		;Bit en cours de traitement

echant	equ	H'10'		;"duree" minimale d'un etat (haut ou bas)
echantd	equ	H'11'		;"duree" correpondant d'un "demi etat"
mini	equ	H'12'		;variable temporaire transfere dans echant

temp1	equ	H'13'
octet	equ	H'14'
mute	equ	H'15'		;Permet de savoir si on est en mode mute ou non

	org	H'00'
	goto	Debut

;ͻ
;On vient ici lorsqu'il y a interruption (permet d'enregistrer et de recreer le code a la bonne vitesse)
;ͼ
	org	H'04'		; le programme s'execute a partir d'ici lorsqu'il y a interruption
	movwf	W_temp		; On sauvegarde W et STATUS pour pouvoir les restituer intacts
	swapf	STATUS,0	; apres le traitement de l'interruption.
	movwf	ST_temp		; On utilise un swap pour ne pas modifier le contenu de STATUS (C,DC,Z)
	bcf	STATUS,RP0	; pour le cas ou il y aurait interruption lorsqu'on est dans la page mem 1

	btfss	PORTA,0		; Est-on en mode enregistrement?
	call	Enr_code	; Oui alors on va au label Enr_code
	btfsc	PORTA,0		; Est on en mode resitution du code?
	call	Rest_code	; Oui alors on va au label Rest_code

	bcf	INTCON,T0IF	; On annule l'interruption

	swapf	ST_temp,0	; On remet STATUS dans le bon sens
	movwf	STATUS		; Et on le resitue
	swapf	W_temp,1	;
	swapf	W_temp,0	; On a restitue W sans modifier le contenu de STATUS
	retfie			; On revient a l'endroit ou le programm s'est interrompu.

;ͻ
;Configuration du PIC
;ͼ
Debut
	bsf	STATUS,RP0	; On passe dans la zone memoire numero 1
	movlw	B'00000001'	; 
	movwf	OPTION_REG	; Predivision par 4 (incrementation de TMR0 toutes les 4micorsecondes)
	movlw	B'00011100'	; 
	movwf	TRISA		; bit a 1 => patte en entree; bit a 0 => patte en sortie
	movlw	B'00000000'	;
	movwf	TRISB		; bit a 1 => patte en entree; bit a 0 => patte en sortie
	bcf	STATUS,RP0	; On passe dans la zone memoire numero 0

	bsf	PORTA,1		; Port serie a 1 par defaut

	clrf	index
	clrf	octet
	clrf	PORTB

	movlw	H'2F'		; L'etat avant la coupure de l'alimentation est restore
	movwf	EEADR		;
	bsf	STATUS,RP0	; 
	bsf	EECON1,RD	; 
	bcf	STATUS,RP0	;
	movf	EEDATA,0	;
	movwf	PORTB		;

	movlw	H'3F'		;
	movwf	EEADR		; Restitution de
	bsf	STATUS,RP0	; la duree minimale d'un etat pour pouvoir regler les interruptions
	bsf	EECON1,RD	; correctement, et ainsi resituer le code a la bonne vitesse
	bcf	STATUS,RP0	;
	movf	EEDATA,0	;
	movwf	echant		;

	btfsc	PORTA,2		; Cette partie de code permet de tester si on appuie sur les
	goto	Tempo		; deux touches en meme temps.
	btfss	PORTA,3		; Si c'est le cas, on va refaire un echantillonage (indispensable
	clrf	EEADR		; a chaque fois que vous changez de telecommande)
				;
	movf	EEADR,1		; 
	btfsc	STATUS,Z	; 
	goto	Echantillonage	; 

Tempo
	movlw	H'40'		; Permet d'indiquer qu'on est en mode de fonctionnement normal
	movwf	EEADR		;

	decfsz	index,1		;
	goto	Tempo		; Petite tempo pour laisser le temps a l'alimentation de se 
	decfsz	octet,1		; stabiliser.
	goto	Tempo		;

Menu
	bcf	PORTA,0		; Led indicatrice de mode (enregistrement ou lecture)
	bsf	PORTA,1		; Port serie a 1 par defaut

	movf	echantd,0	; On va charger TMR0 avec une valeur calculee a partir de echant:
	movwf	TMR0		; echantd = echant - 8 + (255 - echant) / 2
	clrf	index		; index contient le bit en cours de traitement.

	movlw	H'FF'		;
	movwf	echantd		; Tout ceci permet de calculer la valeur de echantd a partir de echant:
	movf	echant,0	; echantd = echant - 8 + (255 - echant) / 2
	subwf	echantd,1	; On cree un interruption plus rapidement lors d'un changement d'etat
	bcf	STATUS,C	; de la LED receptrice
	rrf	echantd,1	; Division de echantd par 2
	movf	echant,0	;
	addwf	echantd,1	;
	movlw	D'008'		;
	subwf	echantd,1	;
	bsf	index,7		;

;ͻ
;On attend l'appui sur une touche
;ͼ
Menu1
	movlw	H'40'		; Si on est en mode normal, on va au label Menu2
	subwf	EEADR,0		; Sinon, on attend l'appui sur la touche 1
	btfsc	STATUS,Z	; (configuration des touches d'allumage, de selection
	goto	Menu2		; de source, d'enceinte, et de mute)
	btfsc	PORTA,2		;
	goto	Menu1		;
Menu2
	btfss	PORTA,2		; Possibilite de selectionner les sources sans la telecommande avec les touches
	call	Select_Entree	
	btfss	PORTA,3		; Idem pour les Hauts Parleurs
	call	Select_Sortie
	btfss	PORTA,4		; La telecommande est telle en train d'emmetre?
	goto	Menu2		; Non alors on repart au label Menu2

	movlw	H'30'		; H'30' -> FSR. FSR est un pointeur sur la RAM. Il contient l'adresse d'une
	movwf	FSR		; case memoire de la RAM. On recupere le contenu de cette case memoire dans
				; la variable INDF.

;ͻ
;On efface une partie de la RAM avant d'enregistrer le code
;ͼ
Efface_RAM
	clrf	INDF		;
	incf	FSR,1		; Va permettre d'effacer le contenu de la RAM entre les adresse H'30'
	movlw	H'50'		; et H'4F'. Cette zone de memoire servira a stocker provisoirement le
	subwf	FSR,0		; code de la telecommande. (On ne peux pas stocker directement le code
	btfss	STATUS,Z	; de la telecommande dans l'EEPROM, car le temps d'ecriture est trop long)
	goto	Efface_RAM	;

	movlw	H'30'		; On recharge FSR avec la valeur H'30'
	movwf	FSR		; 

;ͻ
;On attend que la telecommande commence a emettre
;ͼ
Boucle1
	btfss	PORTA,4		; On attent que la telecommande commence a emmetre.
	goto	Boucle1

	movlw	B'10100000'	; On va autoriser les interruptions dues au Timer0
	movwf	INTCON		; (interruption chaque fois que TMR0 passe de H'FF' a H'00')

	movf	echantd,0
Boucle1_1
	movwf	TMR0		; On attend un front descendant. On va reelement debuter l'enregistrement
	btfsc	PORTA,4		; lorsqu'on aura ce front descendant en provenance de la telecommande
	goto	Boucle1_1	;

	movf	PORTA,0		; preced va contenir la valeur precedente de PORTA (pour pouvoir
	movwf	preced		; detecter les changements d'etat)

;ͻ
;C'est l'enregistement du code. A chaque interruption l'etat de la LED emettrice est stocke dans un bit 
;de la RAM. L'horloge du PIC est resynchronisee sur l'horloge de la telecommande a chaque changement    
;d'etat de cette derniere										 
;ͼ
Boucle2
	movf	preced,0	; On compare la valeur actuelle de PORTA avec la valeur precedente.
	subwf	PORTA,0		; S'il y a changement d'etat de la led receprtrice, alors on va
	btfss	STATUS,Z	; resynchroniser les interruptions sur le front.
	call	Init_timer0	; 

	movf	PORTA,0		;
	movwf	preced		; On transfert PORTA dans preced

	movlw	H'50'		; On regarde si l'enregistrement n'est pas finit (memoire pleine)
	subwf	FSR,0		;
	btfss	STATUS,Z	;
	goto	Boucle2		; Si l'enregistrement est finit, on ne retourne pas au label Boucle2

	bcf	INTCON,GIE	; On desactive les interruptions

	movlw	H'30'		; On recharge le pointeur sur la RAM: FSR avec la valeur
	movwf	FSR		; du debut 


;ͻ
;On transfere eventuellement le contenu de la RAM dans l'EEPROM (On ne peut pas ecrire directement dans     
;l'EEPROM, car le temps d'ecriture est trop long (4ms mini). 						 
;ͼ
	bsf	PORTA,0		; est-on en mode configuration?
	movlw	H'40'		;
	subwf	EEADR,0		; 
	btfsc	STATUS,Z	;
	goto	Boucle3		; Non alors on saute au label boucle3

	clrf	index
	clrf	octet

Eeprom
	movf	INDF,0		; On recupere le contenu de la RAM a l'adresse FSR.
	movwf	EEDATA		; On met ce contenu dans EEDATA
	call	Ecriture	; et on va l'ecrire dans l'EEPROM

	incf	FSR,1		; Sinon on incremente la valeur du pointeur sur la RAM
	incf	EEADR,1		; et on incremente la valeur de l'adresse de l'EEPROM

	movf	EEADR,0		; On regarde si EEADR a atteint sa valeur finale (4 cas possibles), au
	andlw	B'00001111'	; quel cas, on arrete d'enregisterer
	sublw	H'0F'		; Methode: on regarde si le second digit de l'adresse de l'EEPROM est F
	btfsc	STATUS,Z	; 
	goto	Eeprom1		; 

	goto	Eeprom		; Et on va recommencer un cycle d'ecriture
Eeprom1
	decfsz	index,1
	goto	Eeprom1
	decfsz	octet,1
	goto	Eeprom1

	incf	EEADR,1

;ͻ
;On va maintenant transferer le code recu au port serie          
;On cree des interruptions pour obtenir une vitesse de 9600 bauds
;ͼ
Boucle3
	clrf	index

	bsf	PORTA,0
	movlw   D'237'		; En relation avec la vitesse de transmission
	movwf   TMR0
	movlw	H'30'		; On recharge le pointeur sur la RAM: FSR avec la valeur
	movwf	FSR		; du debut 
	movf	INDF,0		; On recupere le contenu de la RAM a l'adresse FSR.
	movwf	octet
	movlw	B'10100000'	; On va autoriser les interruptions dues au Timer0
	movwf	INTCON		; (interruption chaque fois que TMR0 passe de H'FF' a H'00'
Boucle4
	movlw	H'50'		; As t-on atteint la fin de la RAM?
	subwf	FSR,0		; Si oui on va au label Compare_code
	btfsc	STATUS,Z	;
	goto	Compare_code	;

	movlw	D'010'		; index contient le bit en cours de traitement.
	subwf	index,0		; On regarde si on a transmis tous les bits, a savoir
	btfss	STATUS,Z	; le bit de start, les 8 bits de donnes, et le bit de stop
	goto	Boucle4		; 

	clrf	index		;
	incf	FSR,1		; On incremente diverses variables
	movf	INDF,0		; 
	movwf	octet		;
	goto	Boucle4		; 

;ͻ
;Sous programme de comparaison du code recu et du code memorise
;ͼ
Compare_code
	bcf	INTCON,GIE	; On arrete les interruptions

	movlw	H'40'		;
	subwf	EEADR,0		; Si on est en mode configuration on revient au label Menu directement
	btfss	STATUS,Z	;
	goto	Menu		;

	movlw	H'30'
	movwf	FSR
	clrf	EEADR

Compare_code1
	bsf	STATUS,RP0
	bsf	EECON1,RD
	bcf	STATUS,RP0

	movf	INDF,0		;
	subwf	EEDATA,0	; Des qu'un octet de la RAM (qu'on vient de recevoir de la telecommande)
	btfss	STATUS,Z	; ne correspond pas avec un octet de l'EEPROM (l'un des 4 codes enregistre
	goto	Compare_code2	; pour allumer le PC, ...), On passe au code suivant.

	incf	EEADR,1
	incf	FSR,1

	movf	EEADR,0		;
	andlw	B'00001111'	; On teste si on a atteint la fin d'un code (il y en a 4 au total)
	sublw	H'0E'		;
	btfss	STATUS,Z	;
	goto	Compare_code1	;

	movlw	H'0E'		;
	subwf	EEADR,0		;
	btfss	STATUS,Z	;
	goto	Compare_code3	; Est ce que le code emmis par la telecommande correspond a celui
	bsf	PORTB,4		; qui permet d'allumer le P.C. a distance?
	clrf	temp1		; Si oui, on va au label Fin
	clrf	index		; Si non, on va au label Compare_code3
	movlw	D'004'		;
	movwf	octet		;
				;
	movlw	H'40'		;
	movwf	EEADR		;
	goto	Fin		;

Compare_code3			;
	movlw	H'1E'		;
	subwf	EEADR,0		; Est ce que le code emmis par la telecommande correspond a celui
	btfss	STATUS,Z	; qui permet de selectionner les sources audio?
	goto	Compare_code4	; Oui alors on appelle le sous programme Select_Entree, puis on revient au menu
	movlw	H'40'		; Sinon on va au label Compare_code4
	movwf	EEADR		;
	call	Select_Entree	;
	goto	Menu		;
	
Compare_code4
	movlw	H'2E'		;
	subwf	EEADR,0		;
	btfss	STATUS,Z	; Idem que precedemment pour les Hauts parleurs
	goto	Compare_code5	;
	movlw	H'40'		;
	movwf	EEADR		;
	call	Select_Sortie	;
	goto	Menu		;

Compare_code5
	call	Muting		;
	movlw	H'40'		; Idem, avec Mute
	movwf	EEADR		;
	goto	Menu		;

Compare_code2
	movlw	H'30'
	movwf	FSR

	movf	EEADR,0		;
	addlw	H'10'		;
	andlw	B'11110000'	; Tant qu'on n'a pas parcouru toute la memoire EEPROM,
	movwf	EEADR		; on revient au label Compare_code1
	sublw	H'40'		;
	btfss	STATUS,Z	;
	goto	Compare_code1	;
	movlw	H'40'
	movwf	EEADR
	goto	Menu

;ͻ
;Sous programme de selection des sources et des enceintes
;ͼ
Muting
	btfss	mute,0		;
	goto	Muting1		;
	bcf	mute,0		;
	movlw	H'2F'		; Si on est en mode mute, et qu'on rappuie sur la touche,
	movwf	EEADR		; On se remet dans le mode precedent
	bsf	STATUS,RP0	; 
	bsf	EECON1,RD	; 
	bcf	STATUS,RP0	;
	movf	EEDATA,0	;
	movwf	PORTB		;
	return
Muting1
	movlw	B'10001000'	; sinon, on coupe les enceintes et le sources
	movwf	PORTB
	bsf	mute,0
	return

Select_Entree
	movlw	H'40'
	subwf	EEADR,0
	btfss	STATUS,Z
	return			; On ne doit pas etre ici pour le moment (mode configuration)

Select_Entree0			; Tout ce qui suit permet de selectionner les entrees (commute diferents relais)
	btfss	PORTA,2
	goto	Select_Entree0

	btfsc	mute,0
	call	Muting

	btfss	PORTB,5
	goto	Select_Entree1
	bcf	PORTB,5
	bcf	PORTB,6
	bcf	PORTB,7
	goto	Fin_Select
Select_Entree1
	btfss	PORTB,6
	goto	Select_Entree2
	bsf	PORTB,7
	bsf	PORTB,5
	bcf	PORTB,6
	goto	Fin_Select
Select_Entree2
	bsf	PORTB,7
	bcf	PORTB,5
	bsf	PORTB,6
	goto	Fin_Select

Select_Sortie
	movlw	H'40'
	subwf	EEADR,0
	btfss	STATUS,Z
	return			; On ne doit pas etre ici pour le moment (mode configuration)

Select_Sortie0			; Tout ce qui suit permet de configurer les sorties (commute les differents relais)
	btfss	PORTA,3
	goto	Select_Sortie0

	btfsc	mute,0
	call	Muting

	btfss	PORTB,0
	goto	Select_Sortie1
	bcf	PORTB,0
	bcf	PORTB,1
	bcf	PORTB,2
	bcf	PORTB,3
	goto	Fin_Select
Select_Sortie1
	btfss	PORTB,1
	goto	Select_Sortie2
	bsf	PORTB,3
	bcf	PORTB,1
	bcf	PORTB,2
	bsf	PORTB,0
	goto	Fin_Select
Select_Sortie2
	btfss	PORTB,2
	goto	Select_Sortie3
	bsf	PORTB,3
	bcf	PORTB,0
	bcf	PORTB,2
	bsf	PORTB,1
	goto	Fin_Select
Select_Sortie3
	bsf	PORTB,3
	bcf	PORTB,0
	bcf	PORTB,1
	bsf	PORTB,2
	goto	Fin_Select

Fin_Select			;Sauvegarde le l'etat des relais pour pouvoir le restorer lors d'une coupure de l'alimentation
	movlw	H'2F'
	movwf	EEADR
	movf	PORTB,0
	movwf	EEDATA
	call	Ecriture
	movlw	H'40'
	movwf	EEADR
	return
;ͻ
;Sous programme en rapport avec la restitution du code au port serie
;ͼ
Rest_code
	movf	index,1		;
	btfsc	STATUS,Z	; Si index vaut 1, on transmet le bit de start
	goto	StartBit	;

	movlw	D'009'		;
	subwf	index,0		; Si index vaut 9, on transmet le bit de stop
	btfsc	STATUS,Z	;
	goto	StopBit		;

	bcf	STATUS,C	; Sinon on transmet les 8 bits de donnes, 
	rrf	octet,1		; en sachant que ca commence par le bit de poids le
	btfss	STATUS,C	; plus faible!
	bcf	PORTA,1		;
	btfsc	STATUS,C	;
	bsf	PORTA,1		;

Retour_rest
	incf	index,1
	movlw   D'237'
	movwf   TMR0
	return

StartBit
	bcf	PORTA,1		;bit de start => port serie a zero
	goto	Retour_rest

StopBit
	bsf	PORTA,1		;bit de stop => port serie a un
	goto	Retour_rest

;ͻ
;Sous programme en rapport avec l'enregistrement du code
;ͼ
Enr_code
	btfss	PORTA,4		; La LED receptrice est-elle a "1"
	goto	Suite_Enr_code	; Non alors on laisse le bit correpondant de la RAM a "0" (RAM prealablement effacee)
	movf	index,0		; On met index dans W
	iorwf	INDF,1		; On fait un OU logique entre index et le contenude la RAM a l'adresse FSR
				; ceci permet de garder le contenu de INDF intact, et de ne mettre a "1"
				; que le bit qui nous interesse, c'est a dire celui d'index
Suite_Enr_code
	bcf	STATUS,C	; Permet d'eviter l'apparution de "1" lors de la rotation (rrf)
	rrf	index,1		; deplacement de tous les bits d'un cran vers la droite
	btfss	STATUS,C	; On regarde si on a fait un tour complet.
	goto	Term_Enr_code	; Sinon on revient
	clrf	index		; Si oui:
	bsf	index,7		; On reinitialise index a B'10000000'
	incf	FSR,1		; On incremente FSR pour memoriser dans l'octet suivant de la RAM
	goto	Term_Enr_code

Term_Enr_code
	movf	echant,0	; On reinitialise TMR0 avec la valeur contenue dans echant
	movwf	TMR0		; (enchant contient la duree minimale d'un etat haut ou bas)

	return


Fin
	decfsz	temp1,1		; Tempo pour allumer le P.C.
	goto	Fin
	decfsz	index,1
	goto	Fin
	decfsz	octet,1
	goto	Fin

	bcf	PORTB,4
	goto	Menu		; On revient au label Menu

;ͻ
;Sous programme pour ecrire dans la memoire EEPROM
;ͼ
Ecriture
	bsf	STATUS,RP0	; on passe dans la zone memoire numero 1
	clrf	EECON1		; remise a zero des eventuelles erreurs
	bsf	EECON1,WREN	; on autorise l'ecriture

	movlw	H'55'		;
	movwf	EECON2		; securite de microchip pour eviter les ecritures
	movlw	H'AA'		; intempestives.
	movwf	EECON2		;

	bsf	EECON1,WR	; on ecrit pour de bon
Fin_ecriture
	btfsc	EECON1,WR	; 
	goto	Fin_ecriture	; on attend que l'operation d'ecriture soit finie (de l'ordre de la ms,
	bcf	STATUS,RP0	; contre une micro seconde pour un cycle d'horloge)
	return

;ͻ
;Sous programme appele pour mesurer la duree minimale d'un etat haut ou bas de la LED emettrice
;ͼ
Echantillonage
	movlw	D'100'		; On initialise la variable temporaire temp1 a la valeur 100 pour faire
	movwf	temp1		; cent mesures

	bsf	STATUS,RP0	; Zone memoire numero 1
	movlw	B'00000010'	; 
	movwf	OPTION_REG	; predivision par 8 (incrementation de TMR0 toutes les 8 microsecondes)
	bcf	STATUS,RP0	; On revient dans la zone memoire numero 0
;	bsf	PORTA,1		; On allume la LED indicatrice
	movlw	H'FF'		; On initialise mini a la plus grande valeur possible
	movwf	mini		; mini contiendra la "duree" minimale d'un etat (haut ou bas) de la LED receptrice
Ech0
	bcf	PORTA,0		; On eteint l'autre LED indicatrice
	bcf	INTCON,GIE	; On desactive les interruptions
	btfsc	PORTA,4		; On attend que la led receptrice soit au niveau bas
	goto	Ech0
Ech1
	btfss	PORTA,4		; On attend que la LED receptrice soit au niveau haut
	goto	Ech1		; (les quelques lignes precedentes permettent de detecter un front montant)
	clrf	TMR0		; 0 -> TMR0
	bsf	PORTA,0		; On alume l'autre LED indicatrice
	movlw	B'10100000'	; 
	movwf	INTCON		; On reactive les interruptions.
Ech2
	movlw	D'250'		; On s'arrange pour qu'il n'y ait jamais interruption
	subwf	TMR0,0		; En fait TMR0 va uniquement nous servir a mesurer la "duree" d'un etat
	btfsc	STATUS,C	; 
	goto	Ech0		; Dans le cas ou TMR0 depasse D'250', on revient  Ech0

	btfsc	PORTA,4		;
	goto	Ech2		; On revient au label Ech2 tant que l'entree de la led receptrice n'est pas a "0"
Ech3
	movlw	D'250'		;
	subwf	TMR0,0		;
	btfsc	STATUS,C	; Idem que precedemment, mais avec un niveau haut pour l'entree de la 
	goto	Ech0		; receptrice.
				; Encore une fois, ces quelques lignes nous ont permis de detecter un
	btfss	PORTA,4		; front montant
	goto	Ech3		;

	bcf	INTCON,GIE	; On desactive les interruptions

	movf	TMR0,0		;		
	subwf	mini,0		; On compare la valeur de TMR0 avec la valeur minimale prise par cette variable
	btfss	STATUS,C	; Si la valeur de TMR0 < mini, alors on va l'enregistrer
	goto	Fin_ech		; Sinon on va au label Fin_ech

	movf	TMR0,0		; 
	movwf	mini		; TMR0 -> mini

	sublw	H'FF'		;
	addlw	D'006'		; 6 Par ce que ca marchait mieux
	movwf	echant		; 255 - mini + 6 -> echant
				; ainsi on pourra charger TMR0 directement avec la valeur de echant
Fin_ech
	decfsz	temp1,1		; As t-on fait 100 echantillonages?
	goto	Ech0		; Non alors on continue
	
;	bcf	PORTA,1		; On eteint la LED indicatrice pour dire que l'echantillonage est finit
	bsf	STATUS,RP0	; 
	movlw	B'00000001'	;
	movwf	OPTION_REG	; On remet la predivision par 4
	bcf	STATUS,RP0	;
;ͻ
;Dans le dernier octet concernant une touche, on enregistre la vitesse d'execution du code
;ͼ
Type_code
	movlw	H'3F'
	movwf	EEADR
	movf	echant,0	; On va ecrire dans la derniere case memoire concernant une touche
	movwf	EEDATA		; la valeur de echant, c'est a dire la "duree" minimale d'un etat
	call	Ecriture	; (haut ou bas)

	clrf	EEADR
	goto	Menu		; On revient au menu avec la bonne valeur dans echant

Init_timer0
	movf	echantd,0	; echantd (comme echantillonage de depart) -> TMR0
	movwf	TMR0		;
	return			;

;ͻ
;Permet d'ecrire des donnees dans la memoire EEPROM du PIC
;ͼ
	org	H'212F'
	de	B'00000000'	;derniere valeur du PORTB (pour la selection des sources et enceintes)
	org	H'213F'
	de	D'117'		;vitesse d'echantillonage par defaut

	end