;*****************************************************************************************
;modifi 24 juill 04
; 6 avril 2004 / PF
;       PROGRAM:                Sub40b_Z   voir DD_SYNTH.ASM (SOURCE FILE)
;                               DDSYNTHz.DAT voir DD_SYNTH.DAT (OPTIONS & FREQUENCY SETTINGS ETC.)
;       ASSEMBLER:              MPASMWIN v02.70
;       AUTHOR:                 Steven Jones         EMAIL stevejones@picknowl.com.au
;       REVISION HISTORY:
;       VERSION:        DATE            COMMENTS
;       1.40            NOV 05 2001     Fixed a bug that caused incorrect data to be
;                                       sent to the DDS chip when MIN RX DDS FREQ and
;                                       MAX RX DDS FREQ are set to negative numbers.
;*****************************************************************************************

#define miniSimDE




#DEFINE         ENCODER_A       PORTA,0         ; Rotary encoder switch.
#DEFINE         ENCODER_B       PORTA,1         ; Rotary encoder switch.
#DEFINE         SWITCH_A        PORTA,2         ; External switch array . (active low)
#DEFINE         SWITCH_B        PORTA,3         ;                 "
#DEFINE         SWITCH_C        PORTA,4         ;                 "

#DEFINE         LCD_RS          PORTB,1         ; LCD Reg select. 0 = instruct 1 = data
#DEFINE         UNUSED          PORTB,2         ; Unused pic pin.
#DEFINE         LCD_E           PORTB,3         ; LCD Enable.     0 = disable  1 = enable
;                               PORTB,4..7      ; LCD 4 bit data bus.
#DEFINE         DDS_LOAD        PORTB,0         ; AD9850/51 update pin.
#DEFINE         DDS_CLOCK       PORTB,5         ; AD9850/51 write clock pin.
#DEFINE         DDS_DATA        PORTB,7         ; AD9850/51 serial data input pin.

#DEFINE         ENC_A_DEBOUNCED FLAGS,0         ; Encoder A state after debouncing.
#DEFINE         ENC_B_DEBOUNCED FLAGS,1         ; Encoder B state after debouncing.
#DEFINE         OLD_A_DEBOUNCED FLAGS,2         ; Old encoder a state after debouncing.
#DEFINE         LEADING_FLAG    FLAGS,3         ; Leading zero indicator.
#DEFINE         NEGATIVE        FLAGS,4         ; Number to display is negative.
#DEFINE         NEGATIVE_RX     FLAGS,5         ; Min RX DDS Freq is negative.
#DEFINE         TIMEOUT         FLAGS,6         ; If set, go back to main display.
#DEFINE         EEPROM_UPDATE   FLAGS,7         ; If set, update the EEPROM.

#DEFINE         CAL_SCRN        FLAG2,0         ; Set if displaying 10 MHz CAL screen.
#DEFINE         RIT_SCRN        FLAG2,1         ; Set if displaying RIT screen.
#DEFINE         PRESSED         FLAG2,2         ; Result flag from button pressed call.
#DEFINE         WRITE_FLAG      FLAG2,3         ; Set if we want to do an EEPROM write.
#DEFINE         RPT_FLAG        FLAG2,4         ; Set if RPT offset is positive.
#DEFINE         SW_A_FLAG       FLAG2,5         ; Set if step size button is pressed.

;*****************************************************************************************
;               General equates
;*****************************************************************************************


RESETVECTOR     EQU             H'00'           ; PIC Reset vector.
INTVECTOR       EQU             H'04'           ; PIC Interrupt vector.
TRUE            EQU             H'FF'
FALSE           EQU             H'00'
AD9850          EQU             H'00'
AD9851          EQU             H'01'
YES             EQU             H'01'
NO              EQU             H'00'

MIN_ON          EQU              6              ; Min valid button pressed time. (6 x 8mS)
MAX_ON          EQU             70              ; Max valid button pressed time.(70 x 8mS)
TIMEOUT_NUM     EQU              5              ; Display timeout. 10 sec (5 x 2.1 sec)

;*****************************************************************************************
;               RAM variables
;*****************************************************************************************



VERSION_NUMBER          EQU     141             ; Software version. (no decimal point)



	p=16F84
     ORG     H'2100'                 ; PROGRAM OPTIONS, INITIAL PLL & DISPLAY
   

PIC             EQU     84
RAMSTART        EQU     H'0C'
RAM_INT         EQU     H'4E'    

DDS_WORD EQU     H'00'
DDS_MAX  EQU      6
DIG3     EQU     '0'

TEXT_STRING     MACRO   NAMES, TEXT, TERMINATOR         ; Text string macro.
NAMES           EQU $ - TABLE_START
                DT TEXT, TERMINATOR
                ENDM


DIG1    EQU     48 + (VERSION_NUMBER / H'100')
DIG2    EQU     48 + ((VERSION_NUMBER - ((VERSION_NUMBER / H'100') * H'100')) / H'10')


#DEFINE  FILE_VERSION    " DDSYNTHz v"


FILE_STRING     MACRO   NAMES, TEXT, TERMINATOR         ; Text string macro.
NAMES           EQU $ - TABLE_START
                DT TEXT, DIG1, ".", DIG2, DIG3, TERMINATOR
                ENDM

;*****************************************************************************************
;                       Decimal to Hex conversions.
;*****************************************************************************************

EE_RX_DDS_FREQ:         EQU     $ - H'2100'
                        DE     ((RX_DDS_FREQ >> 31) & H'1') * H'FF'
                        DE      (RX_DDS_FREQ >> 24) & H'FF'
                        DE      (RX_DDS_FREQ >> 16) & H'FF'
                        DE      (RX_DDS_FREQ >>  8) & H'FF'
                        DE      (RX_DDS_FREQ      ) & H'FF'
EE_STEP_NUM:            EQU     $ - H'2100'
                        DE      0

EE_TX_OFFSET_FREQ:      EQU     $ - H'2100'
                        DE     ((TX_OFFSET_FREQ >> 31) & H'1') * H'FF'
                        DE      (TX_OFFSET_FREQ >> 24) & H'FF'
                        DE      (TX_OFFSET_FREQ >> 16) & H'FF'
                        DE      (TX_OFFSET_FREQ >>  8) & H'FF'
                        DE      (TX_OFFSET_FREQ      ) & H'FF'


EE_MIN_RX_DDS_FREQ:     EQU     $ - H'2100'
                        DE     ((MIN_RX_DDS_FREQ >> 31) & H'1') * H'FF'
                        DE      (MIN_RX_DDS_FREQ >> 24) & H'FF'
                        DE      (MIN_RX_DDS_FREQ >> 16) & H'FF'
                        DE      (MIN_RX_DDS_FREQ >>  8) & H'FF'
                        DE      (MIN_RX_DDS_FREQ      ) & H'FF'

EE_MAX_RX_DDS_FREQ:     EQU     $ - H'2100'
                        DE     ((MAX_RX_DDS_FREQ >> 31) & H'1') * H'FF'
                        DE      (MAX_RX_DDS_FREQ >> 24) & H'FF'
                        DE      (MAX_RX_DDS_FREQ >> 16) & H'FF'
                        DE      (MAX_RX_DDS_FREQ >>  8) & H'FF'
                        DE      (MAX_RX_DDS_FREQ      ) & H'FF'

EE_MAX_RIT_OFFSET:      EQU     $ - H'2100'
                        DE      0
                        DE      (MAX_RIT_OFFSET >> 24) & H'FF'
                        DE      (MAX_RIT_OFFSET >> 16) & H'FF'
                        DE      (MAX_RIT_OFFSET >>  8) & H'FF'
                        DE      (MAX_RIT_OFFSET      ) & H'FF'

EE_OFFSET_FREQ:         EQU     $ - H'2100'
                        DE      0
                        DE      (OFFSET_FREQ >> 24) & H'FF'
                        DE      (OFFSET_FREQ >> 16) & H'FF'
                        DE      (OFFSET_FREQ >>  8) & H'FF'
                        DE      (OFFSET_FREQ      ) & H'FF'

EE_MAX_DDS_FREQ         EQU     $ - H'2100'
                        DE      0
                        DE      (MAX_DDS_FREQ >> 24) & H'FF'
                        DE      (MAX_DDS_FREQ >> 16) & H'FF'
                        DE      (MAX_DDS_FREQ >>  8) & H'FF'
                        DE      (MAX_DDS_FREQ      ) & H'FF'

EE_DISPLAY_MULT:        EQU     $ - H'2100'
                        DE      MULTIPLIER

DIGIT0   EQU     DDS_CLOCK_FREQ / 4				; 30Mhz
DIGIT1A  EQU     H'4000000'
DIGIT1   EQU     ( DIGIT1A / ( DIGIT0 / 1) )
DIGIT2A  EQU     (( DIGIT1A - ( DIGIT1 * DIGIT0 )) * 16)
DIGIT2   EQU     ( DIGIT2A / DIGIT0 )
DIGIT3A  EQU     (( DIGIT2A - ( DIGIT2 * DIGIT0 )) * 16)
DIGIT3   EQU     ( DIGIT3A / DIGIT0 )
DIGIT4A  EQU     (( DIGIT3A - ( DIGIT3 * DIGIT0 )) * 16)
DIGIT4   EQU     ( DIGIT4A / DIGIT0 )
DIGIT5A  EQU     (( DIGIT4A - ( DIGIT4 * DIGIT0 )) * 16)
DIGIT5   EQU     ( DIGIT5A / DIGIT0 )
DIGIT6A  EQU     (( DIGIT5A - ( DIGIT5 * DIGIT0 )) * 16)
DIGIT6   EQU     ( DIGIT6A / DIGIT0 )
DIGIT7A  EQU     (( DIGIT6A - ( DIGIT6 * DIGIT0 )) * 16)
DIGIT7   EQU     ( DIGIT7A / DIGIT0 )

EE_DDS_CAL:
             EQU     $ - H'2100'             ; (2^56 / DDS cal constant)
             DE      0
             DE      DIGIT1
             DE      DIGIT2 * H'10' + DIGIT3
             DE      DIGIT4 * H'10' + DIGIT5
             DE      DIGIT6 * H'10' + DIGIT7


DDS_CAL_MIN             EQU     554289185               ; (2^56 / 130 MHz) for AD9850.

EE_CAL_MIN              EQU     $ - H'2100'
                        DE      0
                        DE      (DDS_CAL_MIN >> 24) & H'FF'
                        DE      (DDS_CAL_MIN >> 16) & H'FF'
                        DE      (DDS_CAL_MIN >>  8) & H'FF'
                        DE      (DDS_CAL_MIN      ) & H'FF'

DDS_CAL_MAX             EQU     2401919801              ; (2^56 / 30 MHz) constant.
EE_CAL_MAX              EQU     $ - H'2100'
                        DE       0
                        DE      (DDS_CAL_MAX >> 24) & H'FF'
                        DE      (DDS_CAL_MAX >> 16) & H'FF'
                        DE      (DDS_CAL_MAX >>  8) & H'FF'
                        DE      (DDS_CAL_MAX      ) & H'FF'


EE_RPT_OFFSET           EQU     $ - H'2100'
                        DE      0
                        DE      (RPT_OFFSET >> 24) & H'FF'
                        DE      (RPT_OFFSET >> 16) & H'FF'
                        DE      (RPT_OFFSET >>  8) & H'FF'
                        DE      (RPT_OFFSET      ) & H'FF'

;*************************************************************************

; P16F84.INC  Standard Header File, Version 2.00    Microchip Technology, Inc.
;****************************************
W             EQU     H'0000'
F             EQU     H'0001'
;----- Register Files---------------------

INDF          EQU     H'0000'
TMR0          EQU     H'0001'
PCL           EQU     H'0002'
STATUS        EQU     H'0003'
FSR           EQU     H'0004'
PORTA         EQU     H'0005'
PORTB         EQU     H'0006'
EEDATA        EQU     H'0008'
EEADR         EQU     H'0009'
PCLATH        EQU     H'000A'
INTCON        EQU     H'000B'

; OPTION_REG    EQU     H'0081'
; TRISA         EQU     H'0085'
; TRISB         EQU     H'0086'
; EECON1        EQU     H'0088'
; EECON2        EQU     H'0089'

;----- STATUS Bits ----------------------------

IRP           EQU     H'0007'
RP1           EQU     H'0006'
; RP0           EQU     H'0005'
NOT_TO        EQU     H'0004'
NOT_PD        EQU     H'0003'
Z             EQU     H'0002'
DC            EQU     H'0001'
C             EQU     H'0000'

;----- INTCON Bits ----------------------------
GIE           EQU     H'0007'
EEIE          EQU     H'0006'
T0IE          EQU     H'0005'
INTE          EQU     H'0004'
RBIE          EQU     H'0003'
; T0IF          EQU     H'0002'
INTF          EQU     H'0001'
RBIF          EQU     H'0000'
;----- OPTION Bits ----------------------------

NOT_RBPU      EQU     H'0007'
INTEDG        EQU     H'0006'
T0CS          EQU     H'0005'
T0SE          EQU     H'0004'
PSA           EQU     H'0003'
PS2           EQU     H'0002'
PS1           EQU     H'0001'
PS0           EQU     H'0000'
;----- EECON1 Bits ---------------------------

; EEIF          EQU     H'0004'
; WRERR         EQU     H'0003'
; WREN          EQU     H'0002'
; WR            EQU     H'0001'
; RD            EQU     H'0000'

;==============================================
;       RAM Definition
;=============================================

        __MAXRAM H'CF'
        __BADRAM H'07', H'50'-H'7F', H'87'

;==========================================================================
;       Configuration Bits
;==========================================================================

_CP_ON        EQU     H'000F'
_CP_OFF       EQU     H'3FFF'
_PWRTE_ON     EQU     H'3FF7'
_PWRTE_OFF    EQU     H'3FFF'
_WDT_ON       EQU     H'3FFF'
_WDT_OFF      EQU     H'3FFB'
_LP_OSC       EQU     H'3FFC'
_XT_OSC       EQU     H'3FFD'
_HS_OSC       EQU     H'3FFE'
_RC_OSC       EQU     H'3FFF'

;*****************************************************************************************
;       End of P16F84.INC  Standard Header File, Version 2.00   Microchip Technology, Inc.
;*****************************************************************************************

;*****************************************************************************************
;               My definitions that eliminate error 302
;*****************************************************************************************

OPTION_REG    EQU     H'0001'
TRISA         EQU     H'0005'
TRISB         EQU     H'0006'
EECON1        EQU     H'0008'
EECON2        EQU     H'0009'

;*****************************************************************************************
;               My definitions of commonly used register bits
;*****************************************************************************************

#DEFINE         CARRY           STATUS,0
#DEFINE         ZERO            STATUS,2
#DEFINE         RP0             STATUS,5
#DEFINE         T0IF            INTCON,2
#DEFINE         RD              EECON1,0
#DEFINE         WR              EECON1,1
#DEFINE         WREN            EECON1,2
#DEFINE         WRERR           EECON1,3
#DEFINE         EEIF            EECON1,4



ENC_HISTORY	EQU     RAMSTART
COUNT_0	EQU     RAMSTART+01
COUNT_1	EQU     RAMSTART+02
COUNT_2	EQU     RAMSTART+03
COUNT_3	EQU     RAMSTART+04
COUNT_4	EQU     RAMSTART+05
TEMP_1	EQU     RAMSTART+06
TEMP_2	EQU     RAMSTART+07
TEMP_3	EQU     RAMSTART+08
ENC_COUNTER	EQU     RAMSTART+09
INT_COUNT	EQU     RAMSTART+10
SW_B_CNT	EQU     RAMSTART+11
SW_B_OLD	EQU     RAMSTART+12
SW_C_CNT	EQU     RAMSTART+13
SW_BC_CNT	EQU     RAMSTART+14
SW_BC_OLD	EQU     RAMSTART+15
TIMEOUT_L	EQU     RAMSTART+16
TIMEOUT_H	EQU     RAMSTART+17
EEPROM_WRITE	EQU     RAMSTART+18
FLAGS		EQU     RAMSTART+19
FLAG2		EQU     RAMSTART+20
FREQUENCY	EQU     RAMSTART+21
;	EQU     RAMSTART+22
;	EQU     RAMSTART+23
;	EQU     RAMSTART+24
;	EQU     RAMSTART+25
STEP_NUM	EQU     RAMSTART+26
STEP_RATE	EQU     RAMSTART+27
DDS_OLD	EQU     RAMSTART+28
;	EQU     RAMSTART+29
;	EQU     RAMSTART+30
;	EQU     RAMSTART+31
;	EQU     RAMSTART+32
ARG1_10	EQU     RAMSTART+33
ARG1_9	EQU     RAMSTART+34
ARG1_8	EQU     RAMSTART+35
ARG1_7	EQU     RAMSTART+36
ARG1_6	EQU     RAMSTART+37
ARG1_5	EQU     RAMSTART+38
ARG1_4	EQU     RAMSTART+39
ARG1_3	EQU     RAMSTART+40
ARG1_2	EQU     RAMSTART+41
ARG1_1	EQU     RAMSTART+42
ARG1_0	EQU     RAMSTART+43
ARG2_4	EQU     RAMSTART+44
ARG2_3	EQU     RAMSTART+45
ARG2_2	EQU     RAMSTART+46
ARG2_1	EQU     RAMSTART+47
ARG2_0	EQU     RAMSTART+48
ARG3_4	EQU     RAMSTART+49
ARG3_3	EQU     RAMSTART+50
ARG3_2	EQU     RAMSTART+51
ARG3_1	EQU     RAMSTART+52
ARG3_0	EQU     RAMSTART+53
ARG4_4	EQU     RAMSTART+54
ARG4_3	EQU     RAMSTART+55
ARG4_2	EQU     RAMSTART+56
ARG4_1	EQU     RAMSTART+57
ARG4_0	EQU     RAMSTART+58

	

SAVE_STATUS	EQU RAM_INT			;memoire sauvegarde sur IT
SAVE_W_REG	EQU RAM_INT+1
						;  0x4F limit maxi for 16F84	(68 octest RAM)
	
	

                ORG     INTVECTOR

;-----------------------------------------------------------------------------------------
;               The following is processed every 1.024 mS.
;-----------------------------------------------------------------------------------------
                MOVWF   SAVE_W_REG              ; Save W reg.
                SWAPF   STATUS,W
                MOVWF   SAVE_STATUS             ; Save STATUS reg.
                BCF     RP0                     ; Make sure we are addressing bank 0.
;-----------------------------------------------------------------------------------------

;-----------------------------------------------------------------------------------------
;               Debounce the CAL button.
                INCFSZ  SW_B_CNT,W              ; Inc the CAL button count.
                MOVWF   SW_B_CNT                ; But dont inc past 255.
                BTFSC   SWITCH_B                ; Is the switch B pin low ?
                CLRF    SW_B_CNT                ; N. Clear the count.

;-----------------------------------------------------------------------------------------
;               Debounce the TX button.
SW_TX:          INCFSZ  SW_C_CNT,W              ; Inc the TX button count.
                MOVWF   SW_C_CNT                ; But dont inc past 255.
                BTFSC   SWITCH_C                ; Is the switch C pin low ?
                CLRF    SW_C_CNT                ; N. Clear the count.
SW_END:

;-----------------------------------------------------------------------------------------
;               Decrement the Display timeout counter.
                DECF    TIMEOUT_L,F             ; Dec the timeout counter.
                BTFSC   ZERO
                DECF    TIMEOUT_H,F             ; Is it = 0 ?
                BTFSC   ZERO
                BSF     TIMEOUT                 ; Y. Set the timeout flag.
;-----------------------------------------------------------------------------------------
;               Return from interrupt.
;-----------------------------------------------------------------------------------------

RS_INT_STATUS:  SWAPF   SAVE_STATUS,W
                MOVWF   STATUS                  ; Restore STATUS reg.
                SWAPF   SAVE_W_REG,F
                SWAPF   SAVE_W_REG,W            ; Restore W reg without changing STATUS.
                BCF     T0IF                    ; Clear TMR0 interrupt flag.
                RETFIE

;               End of interrupt service routine
;*****************************************************************************************

;*****************************************************************************************
;
;       Text table      (RETLW 'X')
;               Placed in page 0, so we dont have to bother with page bits.
;               Each string terminates with a space with the MSB set, ( ' ' + H'80' )
;               this is displayed as a space. Strings with ( "" ) terminate at the end
;               of the next string.
;
;*****************************************************************************************

TABLE:          ADDWF   PCL,F                   ; Jump to character pointed to in W reg.


;               LABLE           STRING                  TERMINATION
TABLE_START:

 TEXT_STRING    RIT_OFF:  ,     "RIT "                  , ""
 TEXT_STRING    OFFSET:   ,     "OFFSET"                , ' ' + H'80'
 TEXT_STRING    TX:       ,     "TX"                    , ' ' + H'80'
 TEXT_STRING    MIN_RX:   ,     "MIN "                  , ""
 TEXT_STRING    RX:       ,     "RX"                    , ' ' + H'80'
 TEXT_STRING    MAX:      ,     "MAX"                   , ' ' + H'80'
 TEXT_STRING    DDS_FREQ: ,     "DDS "                  , ""
 TEXT_STRING    FREQ:     ,     "FREQ"                  , ' ' + H'80'
 TEXT_STRING    MULTIP:   ,     "MULTIPLIER"            , ' ' + H'80'
 TEXT_STRING    CAL:      ,     "10"                    , ""
 TEXT_STRING    MHZ:      ,     " MHz CAL"              , ' ' + H'80'
 FILE_STRING    VER:      ,     FILE_VERSION            , ' ' + H'80'
TABLE_END:

;*****************************************************************************************
;               Initialise the hardware.
;*****************************************************************************************
                ORG     RESETVECTOR
                BSF     RP0                     ; Select bank 1 for TRIS reg access.
                MOVLW   B'00000001'             ; Set prescaler to TMR0.
                MOVWF   OPTION_REG              ; TMR0 rate = 1:4 of instruction clock.
                GOTO    CONTINUE                ; Jump over interrupt routines
                                                ; and text table.
;*****************************************************************************************
;               The interupt service routines and text table will be placed here.
;*****************************************************************************************
                ORG     TABLE_END

CONTINUE:       CLRF    TRISB                   ; RB0..RB7 = Output.
                BCF     RP0                     ; Return to bank 0 for port access.
                MOVLW   B'10100000'             ; Enable TMR0 overflow interrupt.
                MOVWF   INTCON                  ; Sets interrupt rate to 1.024 mS.
                CLRF    PORTB                   ; Set all pins to LCD & DDS low.
                MOVLW   100
                CALL    MS_WAIT                 ; Wait until the LCD has powered up.
                MOVLW   COUNT_2                 ; Point to start of RAM we want to clear.
                MOVWF   FSR
                MOVLW   SAVE_STATUS - COUNT_2   ; Number of bytes of RAM to clear.
                CALL    CLEAR_BYTES             ; Clear the RAM.
                CALL    INIT_LCD                ; Initialise the LCD display.
                CALL    SEND_DDS_WORD           ; Initialise the DDS chip.
                MOVLW   VER
                CALL    LCD_TEXT                ; Display version info.
VER_LOOP:       BTFSS   EEPROM_WRITE,7          ; Wait 1 sec. (until EEPROM_WRITE > 127)
                GOTO    VER_LOOP
                BTFSS   SWITCH_B                ; Is the CAL button pressed.
                GOTO    CAL_10MHZ               ; Y. Display the calibration screens.

;********************************************************************************
;               Start of subroutines
;*****************************************************************************************
;
;      NAME:    ADD_TX_OFFSET
;   PURPOSE:    Add the TX offset (from EEPROM) to ARG2.
;     INPUT:    ARG2 = frequency.
;    OUTPUT:    ARG2 = frequency + TX offset.
;               (ARG3 destroyed)
;*****************************************************************************************

ADD_TX_OFFSET:  MOVLW   EE_TX_OFFSET_FREQ       ; Get the TX offset.
                CALL    EE_TO_ARG3              ; Fall through to ADD,
                                                ; and return via ADD.

;*****************************************************************************************
;     NAMES:    ADD                     ARG2 = ARG2 + ARG3.     (all routines are 40 bit)
;               SUB                     ARG2 = ARG2 - ARG3.
;               NEG_ARG2                ARG2 = -ARG2.
;               NEG_ARG3                ARG3 = -ARG3.
;               CLEAR_ARG1              ARG1 = 0.
;               CLEAR_ARG2              ARG2 = 0.
;               CLEAR_ARG3              ARG3 = 0.
;*****************************************************************************************

ADD:            MOVLW   ARG1_9                  ; ARG1 = ARG3.
                CALL    COPY_FROM_ARG3
ADDER:          CALL    ADD_ARG1_ARG2           ; ARG1 = ARG1 + ARG2 = ARG3 + ARG2.
ARG1_TO_ARG2:   MOVLW   ARG1_9                  ;
                GOTO    COPY_TO_ARG2            ; ARG2 = ARG2 + ARG3.

SUB:            CALL    NEG_ARG3_ARG1           ; ARG1 = -ARG3.
                GOTO    ADDER

NEG_ARG3:       CALL    NEG_ARG3_ARG1
ARG1_TO_ARG3:   MOVLW   ARG1_9
                GOTO    COPY_TO_ARG3            ; ARG3 = -ARG3.

NEG_ARG2:       MOVLW   ARG1_9                  ; COPY ARG2 TO ARG1.
                CALL    COPY_FROM_ARG2
                CALL    NEG_ARG1
                GOTO    ARG1_TO_ARG2            ; ARG2 = -ARG2.

NEG_ARG3_ARG1:  MOVLW   ARG1_9                  ; ARG1 = ARG3.
                CALL    COPY_FROM_ARG3
NEG_ARG1:       COMF    ARG1_9,F                ;   (complement ARG1)
                COMF    ARG1_8,F
                COMF    ARG1_7,F
                COMF    ARG1_6,F
                COMF    ARG1_5,F
                INCFSZ  ARG1_5,F                ;   (add 1 to ARG1)
                RETURN
                INCFSZ  ARG1_6,F
                RETURN
                INCFSZ  ARG1_7,F
                RETURN
                INCFSZ  ARG1_8,F
                RETURN
                INCF    ARG1_9,F
                RETURN                          ; ARG1 = -ARG3.


ADD_ARG1_ARG2:  MOVF    ARG2_0,W                ; ARG1_9..5 = ARG1_9..5 + ARG2_4..0
                ADDWF   ARG1_5,F                ; Add the 1st (LSD) bytes.
                MOVF    ARG2_1,W
                BTFSC   CARRY
                INCFSZ  ARG2_1,W                ; If there was a carry, inc the next byte.

                ADDWF   ARG1_6,F                ; Add the 2nd bytes.
                MOVF    ARG2_2,W
                BTFSC   CARRY
                INCFSZ  ARG2_2,W                ; If there was a carry, inc the next byte.

                ADDWF   ARG1_7,F                ; Add the 3rd bytes.
                MOVF    ARG2_3,W
                BTFSC   CARRY
                INCFSZ  ARG2_3,W                ; If there was a carry, inc the next byte.

                ADDWF   ARG1_8,F                ; Add the 4th bytes.
                MOVF    ARG2_4,W
                BTFSC   CARRY
                INCFSZ  ARG2_4,W                ; If there was a carry, inc the next byte.

                ADDWF   ARG1_9,F                ; Add the 5th bytes.
                RETURN



CLEAR_ARG1:     CALL    SET_ARG1                ; Set FSR = ARG1_10, COUNT_1 = 11.
                GOTO    CLEAR_LOOP

CLEAR_ARG2:     MOVLW   ARG2_4
                GOTO    CLEAR

CLEAR_ARG3:     MOVLW   ARG3_4
CLEAR:          MOVWF   FSR
                MOVLW   5
CLEAR_BYTES:    MOVWF   COUNT_1                 ; Save the count.
CLEAR_LOOP:     CLRF    INDF                    ; Clear the RAM byte.
                INCF    FSR,F                   ; Inc the RAM address pointer.
                DECFSZ  COUNT_1,F               ; Dec the count.
                GOTO    CLEAR_LOOP              ; Loop until all bytes cleared.
                RETURN

SET_ARG1:       MOVLW   ARG1_10                 ; Set FSR = ARG1_10, COUNT_1 = 11.
                MOVWF   FSR
                MOVLW   11
                MOVWF   COUNT_1
                RETURN

;*****************************************************************************************
;
;      NAME:    CMP
;
;   PURPOSE:    40 bit compare.  ARG2 - ARG3
;
;     INPUT:    40 bit argment ARG2.
;               40 bit argment ARG3.
;
;    OUTPUT:    None.
;               Carry flag clear (borrow) if result was negative.
;               Zero flag set if equal.
;               (ARG2 & ARG3 unchanged)
;
;*****************************************************************************************

CMP:            CALL    OFFSET_ARGS             ; Add a large offset to ARG2 & ARG3.
                                                ; (makes sure we are dealing )
                                                ; (with positive numbers only)

                CALL    NEG_ARG3_ARG1           ; ARG1 = -ARG3
                CALL    ADD_ARG1_ARG2           ; ARG1 = ARG2 - ARG3

OFFSET_ARGS:    MOVLW   B'10000000'             ; Remove the offset from ARG2 and ARG3.
                XORWF   ARG2_4,F
                XORWF   ARG3_4,F
                MOVF    ARG1_9,W                ; See if the result was 0.
                IORWF   ARG1_8,W                ;
                IORWF   ARG1_7,W                ;
                IORWF   ARG1_6,W                ;
                IORWF   ARG1_5,W                ;
                RETURN                          ; (sets zero flag if result was 0)

;*****************************************************************************************
;
;      NAME:    MINIMUM
;
;   PURPOSE:    40 bit minimum. Return the smallest of two argments.
;
;       INPUT:  40 bit argment in ARG2.
;               40 bit argment in ARG3.
;
;       OUTPUT: The smallest argment is returned in ARG2.
;               (ARG3 unchanged)
;
;*****************************************************************************************

MINIMUM:        CALL    CMP
                BTFSC   CARRY                   ; Is ARG2 > ARG3 ?
                CALL    ARG3_TO_ARG2            ; Y. Set ARG2 the same as ARG3.
                RETURN                          ; N. No change required.

;*****************************************************************************************
;
;      NAME:    MAXIMUM
;
;   PURPOSE:    40 bit maximum. Return the largest of two argments.
;
;       INPUT:  40 bit argment in ARG2.
;               40 bit argment in ARG3.
;
;       OUTPUT: The largest argment is returned in ARG2.
;               (ARG3 unchanged)
;
;*****************************************************************************************

MAXIMUM:        CALL    CMP
                BTFSS   CARRY                   ; Is ARG2 > ARG3 ?
                CALL    ARG3_TO_ARG2            ; N. Set ARG2 the same as ARG3.
                RETURN                          ; Y. No change required.

;*****************************************************************************************
;
;      NAME:    UP_DOWN_RX
;
;   PURPOSE:    If the rotary encoder has been moved, increase or decrease the
;               FREQUENCY variable by step size. Make sure it stays within limit.
;               Also see if the user wants to change the step size.
;
;     INPUT:    None. (freq in the FREQUENCY variable)
;
;    OUTPUT:    ARG2 = frequency +- N. (result also placed back in FREQUENCY variable)
;
;*****************************************************************************************

UP_DOWN_RX:     MOVLW   FREQUENCY
                CALL    COPY_TO_ARG2            ; Get the FREQUENCY variable.

                CALL    UP_DOWN                 ; Chk rotary encoder & adjust step size.
                                                ; (inc or dec ARG2)
                MOVLW   EE_MIN_RX_DDS_FREQ
                CALL    EE_TO_ARG3              ; Get the lower limit.

                CALL    MAXIMUM                 ; Make sure we dont go below that limit.

                MOVLW   EE_MAX_RX_DDS_FREQ
                CALL    EE_TO_ARG3              ; Get the upper limit.

MIN_TO_FREQ:    CALL    MINIMUM                 ; Make sure we dont go above that limit.

                MOVLW   FREQUENCY               ; Save result to the FREQUENCY variable.
                GOTO    COPY_FROM_ARG2          ; Return via "GOTO".

;*****************************************************************************************
;
;      NAME:    LIMIT_RIT
;
;   PURPOSE:    When in the RIT display, make sure the RIT frequency (FREQUENCY variable)
;               stays within the limit set by EE_MAX_RIT_OFFSET.
;
;     INPUT:    None. (freq in the FREQUENCY variable)
;
;    OUTPUT:    ARG2 = frequency. (result also placed back in FREQUENCY variable)
;
;*****************************************************************************************

LIMIT_RIT:      MOVLW   EE_RX_DDS_FREQ
                CALL    EE_TO_ARG2              ; Prepare the upper limit.

                MOVLW   EE_MAX_RIT_OFFSET
                CALL    EE_TO_ARG3              ; Prepare the upper limit.

                CALL    ADD                     ; ARG2 = RX DDS freq + max RIT offset.

                CALL    PUSH_ARG2               ; Save the result. (upper limit)

                CALL    SUB                     ; ARG2 = RX DDS freq
                CALL    SUB                     ; ARG2 = RX DDS freq - max RIT offset.
                                                ;      = (lower limit)
                MOVLW   FREQUENCY               ;
                CALL    COPY_TO_ARG3            ; Get the FREQUENCY variable.

                CALL    MAXIMUM                 ; Make sure we dont go below the limit.

                CALL    PULL_ARG3               ; Get the upper limit.

                                                ; Make sure we dont go above that limit.
                GOTO    MIN_TO_FREQ             ; Save result to the FREQUENCY variable.
                                                ; Return via "GOTO".

;*****************************************************************************************
;
;      NAME:    UP_DOWN
;
;   PURPOSE:    If the rotary encoder has been moved, inc or dec the variable in ARG2.
;               Also see if the user wants to change the step size.
;
;     INPUT:    ARG2.
;
;    OUTPUT:    ARG2 = ARG2 +- N.
;
;*****************************************************************************************

UP_DOWN:        CALL    CHK_STEP_SIZE           ; Does the user want to change step size.
UP_DOWN_A:      MOVF    ENC_COUNTER,W           ; Any change to the rotary encoder ?
                BTFSC   ZERO                    ; N. Just return.
                RETURN

                MOVWF   COUNT_2                 ; Save the encoder count.
                CLRF    ENC_COUNTER             ; Reset the encoder position count.

                MOVF    STEP_NUM,W              ; Get the step number.
                CALL    TEN_TO_POWER_W          ; Convert to step size, place it in ARG3.

                BTFSS   COUNT_2,7               ; Is the encoder count negative ?
                GOTO    STEP_UP

                CALL    NEG_ARG3                ; Y. Make step size negative.
                COMF    COUNT_2,F               ;    Make encoder count positive.
                INCF    COUNT_2,F

STEP_UP:        CALL    ADD                     ; Inc ARG2 by the step size.
                DECFSZ  COUNT_2,F               ; Dec the loop count.
                GOTO    STEP_UP                 ; Inc ARG2 until count = 0.
                RETURN

;*****************************************************************************************
;
;      NAME:    TEN_TO_POWER_W
;                            (W)
;   PURPOSE:    Calculate 10^
;
;     INPUT:    W = Power.
;                         (W)
;    OUTPUT:    ARG3 = 10^
;
;*****************************************************************************************

TEN_MEG:        MOVLW   7
TEN_TO_POWER_W: ADDLW   1
                MOVWF   COUNT_3                 ; Save the power + 1.
                CALL    CLEAR_ARG3
                INCF    ARG3_0,F                ; Set ARG3 to 1.
                GOTO    END_TEST
POWER_LOOP:     MOVLW   10
                CALL    ARG3_TIMES_W
END_TEST:       DECFSZ  COUNT_3,F               ; Keep mult by ten until counter = 0.
                GOTO    POWER_LOOP
                RETURN

;*****************************************************************************************
;
;      NAME:    ARG3_TIMES_W
;
;   PURPOSE:    Calculate ARG3 (+ Numbers only) x W
;
;     INPUT:    W = Multiplier.
;
;    OUTPUT:    ARG3 = ARG3 x W.
;
;*****************************************************************************************

ARG3_TIMES_W:   MOVWF   COUNT_0                 ; Save multiplier.
                CALL    PUSH_ARG2               ; Save ARG2.
                CALL    CLEAR_ARG2              ;
                MOVF    COUNT_0,W               ; Get multiplier.
                MOVWF   ARG2_0                  ; Place it in ARG2.
                CALL    MULTIPLY                ; Do the multiply.
                CALL    PULL_ARG2               ; Recover ARG2.
                MOVLW   ARG1_4                  ; Return the result in ARG3.
                GOTO    COPY_TO_ARG3            ; Return via "GOTO".



;*****************************************************************************************
;
;     NAMES:    COPY_TO_ARG2            Copy 5 bytes pointed to by W to ARG2.
;               COPY_TO_ARG3            Copy 5 bytes pointed to by W to ARG3.
;               COPY_FROM_ARG2          Copy ARG2 to the address pointed to by W.
;               COPY_FROM_ARG3          Copy ARG3 to the address pointed to by W.
;               ARG2_TO_ARG3            Copy ARG2 to ARG3.
;               ARG3_TO_ARG2            Copy ARG3 to ARG2.
;               PUSH_ARG2               Copy ARG2 to TEMP ARG.
;               PUSH_ARG3               Copy ARG3 to TEMP ARG.
;               PULL_ARG2               Copy TEMP ARG to ARG2.
;               PULL_ARG3               copy TEMP ARG to ARG3.
;
;   PURPOSE:    Move 5 bytes of RAM from source address to dest address.
;               (starting at source and dest, then source+1 dest+1, etc.)
;
;               RAM usage       TEMP_1 = destination address.
;                               TEMP_2 = source address.
;                               TEMP_3 = byte to move.
;
;*****************************************************************************************

PUSH_ARG2:      MOVLW   ARG4_4                  ; \
COPY_FROM_ARG2: MOVWF   TEMP_1                  ;  \
                MOVLW   ARG2_4                  ;   \
                GOTO    COPY_B                  ;    |
                                                ;    |
PUSH_ARG3:      MOVLW   ARG4_4                  ;    |
COPY_FROM_ARG3: MOVWF   TEMP_1                  ;    |
                MOVLW   ARG3_4                  ;    |
COPY_B:         MOVWF   TEMP_2                  ;    |
                GOTO    COPY_A                  ;    |
                                                ;    |
ARG3_TO_ARG2:   MOVLW   ARG3_4                  ;    |
                GOTO    COPY_TO_ARG2            ;     \  set the various source and
                                                ;     /  destination addresses.
PULL_ARG2:      MOVLW   ARG4_4                  ;    |
COPY_TO_ARG2:   MOVWF   TEMP_2                  ;    |
                MOVLW   ARG2_4                  ;    |
                GOTO    COPY                    ;    |
                                                ;    |
ARG2_TO_ARG3:   MOVLW   ARG2_4                  ;    |
                GOTO    COPY_TO_ARG3            ;    |
                                                ;    |
PULL_ARG3:      MOVLW   ARG4_4                  ;   /
COPY_TO_ARG3:   MOVWF   TEMP_2                  ;  /
                MOVLW   ARG3_4                  ; /

COPY:           MOVWF   TEMP_1                  ;
COPY_A:         MOVLW   5                       ; 5 bytes to move.
                MOVWF   COUNT_1
COPY_LOOP:      MOVF    TEMP_2,W                ; Get source address.
                MOVWF   FSR                     ;        "
                MOVF    INDF,W                  ; Get the byte from the source address.
                MOVWF   TEMP_3                  ;                "
                MOVF    TEMP_1,W                ; Get destination address.
                MOVWF   FSR                     ;          "
                MOVF    TEMP_3,W                ; Place the byte at destination address.
                MOVWF   INDF                    ;                   "
                INCF    TEMP_1,F                ; Inc dest address.
                INCF    TEMP_2,F                ; Inc source address.
                DECFSZ  COUNT_1,F
                GOTO    COPY_LOOP               ; Loop until all bytes copied.
                RETURN


;*****************************************************************************************
;
;      NAME:    WRITE_EEPROM
;
;   PURPOSE:    Write the byte of RAM pointed to by FSR,
;               to the EEPROM address pointed to by EEADR.
;               If the RAM and EEPROM data are the same, the EEPROM is not reprogrammed.
;               Assumes the source address FSR has been set,
;               and the destination address EEADR has been set.
;
;     INPUT:    None.
;
;    OUTPUT:    None.
;
;*****************************************************************************************

WRITE_MULT:     MOVLW   EE_DISPLAY_MULT         ; Dest address is EE_DISPLAY_MULT.
 IF PIC == 84
                MOVWF   EEADR                   ; Save the dest address.
 ELSE
                BSF     RP0                     ; Select bank 1 for EEPROM reg access.
                MOVWF   EEADR                   ; Save the dest address.
                BCF     RP0                     ; Return to bank 0 for port access.
 ENDIF
                MOVLW   ARG2_0                  ; Source address is ARG2.
                MOVWF   FSR                     ; Save it in the indirect register.

WRITE_EEPROM:   CALL    READ_EEPROM             ; Get the current byte from EEPROM.
                SUBWF   INDF,W
                BTFSC   ZERO                    ; Is it the same as it is in RAM ?
                GOTO    NO_CHANGE               ; Y. No need to update this byte.
                MOVF    INDF,W                  ; N. Get the data byte from RAM.
 IF PIC == 84
                MOVWF   EEDATA                  ;    Put it in the buffer.
                BSF     RP0                     ;    Select bank 1 for EEPROM reg access.
                BCF     EEIF                    ;    Clear the EEPROM write done flag.
 ELSE
                BCF     EEIF                    ;    Clear the EEPROM write done flag.
                BSF     RP0                     ;    Select bank 1 for EEPROM reg access.
                MOVWF   EEDATA                  ;    Put it in the buffer.
 ENDIF
                BSF     WREN                    ;    Enable EEPROM write.
                BCF     INTCON,GIE              ;    Disable interupts.
                MOVLW   H'55'                   ;    Perform required safety steps.
                MOVWF   EECON2
                MOVLW   H'AA'
                MOVWF   EECON2
                BSF     WR                      ;    Begin the write.
                BSF     INTCON,GIE              ;    Enable interupts.
 IF PIC == 628
                BCF     RP0                     ;    Return to bank 0 for port access.
 ENDIF
WRITE_LOOP:     BTFSS   EEIF                    ;    Loop until the write is completed.
                GOTO    WRITE_LOOP
 IF PIC == 84
                BCF     RP0                     ;    Return to bank 0 for port access.
 ENDIF
NO_CHANGE:      BCF     EEPROM_UPDATE           ; Clear the update required flag.
                RETURN

;*****************************************************************************************
;
;      NAME:    READ_EEPROM
;
;   PURPOSE:    Read the byte of EEPROM data pointed to by EEADR.
;               Assumes the read address EEADR has been set.
;
;     INPUT:    None.
;
;    OUTPUT:    W = The byte read from EEPROM.
;
;*****************************************************************************************

READ_MULT:      MOVLW   EE_DISPLAY_MULT         ; Source address is EE_DISPLAY_MULT.
 IF PIC == 84
                MOVWF   EEADR                   ; Save the source address.
 ELSE
                BSF     RP0                     ; Select bank 1 for EEPROM reg access.
                MOVWF   EEADR                   ; Save the source address.
                BCF     RP0                     ; Return to bank 0 for port access.
 ENDIF

READ_EEPROM:    BSF     RP0                     ; Select bank 1 for EEPROM reg access.
                BSF     RD                      ; Perform an EEPROM read.
 IF PIC == 84
                BCF     RP0                     ; Return to bank 0 for port access.
                MOVF    EEDATA,W                ; Get the read byte.
 ELSE
                MOVF    EEDATA,W                ; Get the read byte.
                BCF     RP0                     ; Return to bank 0 for port access.
 ENDIF
                RETURN

;*****************************************************************************************
;
;      NAME:    MULTIPLY
;
;   PURPOSE:    40 bit * 40 bit unsigned multiply.    (ARG1 = ARG2 x ARG3)
;
;     INPUT:    ARG2_4..0  (MSD..LSD) (+ or - numbers )
;               ARG3_4..0  (MSD..LSD) (+ numbers only )
;
;    OUTPUT:    ARG1_9..0  (MSD..LSD)
;               (ARG2 and ARG3 unchanged)
;
;*****************************************************************************************

MULTIPLY:       BTFSS   ARG2_4,7                ; Is ARG2 negative ?
                GOTO    MULT                    ; N. Do the multiply.
                CALL    PUSH_ARG2               ; Y. Save ARG2.
                CALL    NEG_ARG2                ;    Convert to positive.
                CALL    MULT                    ;    Do the multiply.
                GOTO    PULL_ARG2               ;    Get the original ARG2 back.
                                                ;    Return via "GOTO".
MULT:           CALL    CLEAR_ARG1
                MOVLW   40                      ; 40 bits to multiply.
                MOVWF   COUNT_1                 ;
MLOOP:          RRF     ARG3_0,W                ; Rotate the multiplier right 1 bit
                RRF     ARG3_4,F                ; into the carry bit.
                RRF     ARG3_3,F                ;
                RRF     ARG3_2,F                ; ( after 40 rotates ARG3 will be )
                RRF     ARG3_1,F                ; ( returned to its original value )
                RRF     ARG3_0,F                ;

                BTFSC   CARRY                   ; If carry bit is set, add ARG2 to partial
                CALL    ADD_ARG1_ARG2           ; product. ARG1_9..5 = ARG1_9..5 + ARG2_4..0.

                RRF     ARG1_9,F                ; Right shift the partial product.
                RRF     ARG1_8,F                ;
                RRF     ARG1_7,F                ;
                RRF     ARG1_6,F                ;
                RRF     ARG1_5,F                ;
                RRF     ARG1_4,F                ;
                RRF     ARG1_3,F                ;
                RRF     ARG1_2,F                ;
                RRF     ARG1_1,F                ;
                RRF     ARG1_0,F                ;

                DECFSZ  COUNT_1,F               ; All 40 bits done ?
                GOTO    MLOOP                   ; N. Keep going.
                RETURN                          ; Y. All done.


;*****************************************************************************************
;
;      NAME:    BIN_TO_DEC
;
;   PURPOSE:    Convert a 40 bit binary number to a 11 digit ASCII decimal number.
;               First the 40 bit binary number is converted to 11 digit decimal.
;               It is then converted to ASCII, including decimal point, leading zero
;               blanking and negative sign (if NEGATIVE flag set), ready for display on
;               the LCD.
;               (if used on 10 MHz CAL screen, do not include the decimal point.)
;
;     INPUT:    ARG2 = frequency. (+ numbers only )
;
;    OUTPUT:    11 digit ASCII decimal number in ARG1_10..0. (MSD..LSD)
;               (ARG2 unchanged)
;
;*****************************************************************************************

BIN_TO_DEC:     CALL    CLEAR_ARG1
                MOVLW   40
                MOVWF   COUNT_2                 ; 40 bits to process.
                GOTO    SHIFT_TO_DEC

LOOP40:         CALL    SET_ARG1                ; Set FSR = ARG1_10, COUNT_1 = 11.
DEC_LOOP:       MOVLW   3
                ADDWF   INDF,W                  ; If num + 3 > 7 then num = num + 3.
                MOVWF   TEMP_1                  ; Add 3 to num. (adds 6, after next shift)
                BTFSS   TEMP_1,3                ; Is result > 7 ? (bit 3 set)
                GOTO    NO_ADJ
                ADDLW   B'01111000'             ; Y. Move bit 3 to bit 7.
                MOVWF   INDF                    ;    Put the number back in the buffer.
NO_ADJ:         INCF    FSR,F                   ; Point to next byte in buffer.
                DECFSZ  COUNT_1,F
                GOTO    DEC_LOOP                ; Loop until all 11 bytes done.

SHIFT_TO_DEC:   RLF     ARG2_4,W                ; Shift 40 bit number left by one bit
                RLF     ARG2_0,F                ; into dec buffer.
                RLF     ARG2_1,F
                RLF     ARG2_2,F                ; After 40 shifts ARG2 unchanged.
                RLF     ARG2_3,F
                RLF     ARG2_4,F                ; MSD of ARG2
                RLF     ARG1_0,F                ; LSD of ARG1
                RLF     ARG1_1,F
                RLF     ARG1_2,F
                RLF     ARG1_3,F
                RLF     ARG1_4,F
                RLF     ARG1_5,F
                RLF     ARG1_6,F
                RLF     ARG1_7,F
                RLF     ARG1_8,F
                RLF     ARG1_9,F
                RLF     ARG1_10,F               ; MSD of ARG1.
                DECFSZ  COUNT_2,F
                GOTO    LOOP40                  ; Loop until all 40 bits processed.

                BCF     LEADING_FLAG            ; Blank leading zeros until flag is set.

                CALL    SET_ARG1                ; Set FSR = ARG1_10, COUNT_1 = 11.
ASCII_LOOP:     MOVF    INDF,W                  ; Get the number.
                BTFSC   ZERO                    ; Is it = 0 ?
                GOTO    BLANK
NO_BLANK        ADDLW   H'30'                   ; N. Convert it to ASCII.
                MOVWF   INDF
                BSF     LEADING_FLAG            ;    Set the leading blank flag,
                GOTO    NUM_DONE                ;    following zeros will not be blanked.
BLANK:          BTFSC   LEADING_FLAG            ; Y. Is the leading blank flag set.
                GOTO    NO_BLANK                ;    Y. Convert it to ASCII.
                MOVLW   ' '
                MOVWF   INDF                    ;    N. Blank the zero.
                MOVF    FSR,W                   ;       Save the location of the blank
                MOVWF   COUNT_2                 ;       so that a - sign can be inserted.
NUM_DONE:       INCF    FSR,F                   ; Point to the next byte.
                MOVLW   8
                SUBWF   COUNT_1,W               ; Time to cancel leading blanking?
                BTFSC   ZERO
                BSF     LEADING_FLAG            ; Y. Always display char before DP.
                DECFSZ  COUNT_1,F
                GOTO    ASCII_LOOP

                BTFSS   NEGATIVE                ; Was this a negative number?
                RETURN                          ; N. Just return.
                MOVF    COUNT_2,W               ; Y. Get position of last blank.
                MOVWF   FSR
                MOVLW   '-'                     ;    Place - sign in front of the number.
                MOVWF   INDF
                RETURN

;*****************************************************************************************
;
;      NAME:    FREQ_TO_LCD
;
;   PURPOSE:    Update the DDS chip if required, then add the offset freq and multiply
;               by DISP_MULT and display it on the LCD.
;
;     INPUT:    ARG2 = frequency.
;
;    OUTPUT:    None.
;               (ARG1 & ARG3 destroyed)
;
;*****************************************************************************************

FREQ_TO_LCD:    MOVLW   EE_DDS_CAL
                CALL    EE_TO_ARG3              ; Get the XTAL cal constant. (2^56/XTAL)
                CALL    CHK_DDS                 ; Update the DDS chip if required.

                MOVLW   EE_OFFSET_FREQ
                CALL    EE_TO_ARG3              ; Get the offset freq.
                CALL    ADD                     ; Add the offset.

                CALL    CLEAR_ARG3              ; Move the mult byte from EEPROM to ARG3.
                CALL    READ_MULT
                MOVWF   ARG3_0

                CALL    MULTIPLY                ; Do the multiply.
                MOVLW   ARG1_4                  ; Place the result in ARG2.
                CALL    COPY_TO_ARG2            ; Display the frequency, by falling
                                                ; through to DISPLAY_FREQ.

;*****************************************************************************************
;
;      NAME:    DISPLAY_FREQ
;
;   PURPOSE:    Display the frequency in ARG2 on the LCD.
;               Including negative sign, decimal point and leading zero blanking.
;               (if used on 10 MHz CAL screen, do not include the decimal point.)
;
;     INPUT:    ARG2 = frequency.
;
;    OUTPUT:    None.
;
;*****************************************************************************************

DISPLAY_FREQ:   BCF     NEGATIVE                ; Clear the negative flag.
                BTFSS   ARG2_4,7                ; Is ARG2 a negative number?
                GOTO    DISP_FREQ               ; N. Just continue.
                BSF     NEGATIVE                ; Y. Set the negative flag.
                CALL    NEG_ARG2                ;    Convert it to a + number.
DISP_FREQ:      CALL    BIN_TO_DEC              ; Convert the number to ASCII decimal.
                MOVLW   H'C0'                   ; Position the cursor, line 2 pos 1.
                BTFSC   CAL_SCRN                ; Are we in the CAL 10MHz screen ?
                MOVLW   H'C2'                   ; Y. Shift the display to line 2 pos 3.
                CALL    LCD_CMD
                CALL    SET_ARG1                ; Set FSR = ARG1_10, COUNT_1 = 11.
DISP_LOOP:      MOVF    INDF,W                  ; Get char pointed to by FSR.
                CALL    LCD_CHR                 ; Display the char.
                BTFSC   CAL_SCRN                ; Are we in the CAL 10MHz screen ?
                GOTO    NO_DP
                MOVLW   7
                SUBWF   COUNT_1,W               ; N. Is it time to insert a decimal point?
                BTFSC   ZERO
                CALL    LCD_DP                  ;    Y. Print a decimal point.
NO_DP:          INCF    FSR,F                   ; Inc the pointer.
                DECFSZ  COUNT_1,F               ; All done ?
                GOTO    DISP_LOOP               ; N. Continue.

                BTFSC   NEGATIVE                ; Was ARG2 negative ?
                CALL    NEG_ARG2                ; Y. Return it to a negative number.

                BTFSC   CAL_SCRN                ; Are we in the CAL 10MHz screen ?
                RETURN                          ; Y. All done.
                MOVLW   MHZ
                GOTO    LCD_TEXT                ; N. Display ' MHz'.
                                                ;    Return via "GOTO".

;*****************************************************************************************
;
;      NAME:    INIT_LCD
;
;   PURPOSE:    Initialize the 16 x 2 liquid crystal display.
;               The LCD controller chip must be equivalent to the HITACHI 44780.
;
;     INPUT:    None.
;
;    OUTPUT:    None.
;
;*****************************************************************************************

INIT_LCD:       CALL    LCD_RESET               ; Reset the LCD.
                CALL    LCD_RESET               ; Reset the LCD.
                CALL    LCD_RESET               ; Reset the LCD.

                MOVLW   B'00101000'             ; LCD 4 bit mode command.
                MOVWF   PORTB                   ; Also sets LCD_E lead high.
                BCF     LCD_E
                CALL    WAIT_200US
                MOVLW   B'00101000'             ; Set LCD to 2 line, 5x7 dot.
                CALL    LCD_CMD
                MOVLW   B'00001000'             ; Turn the display off.
                CALL    LCD_CMD
                CALL    LCD_CLEAR               ; Clear the display.
                MOVLW   B'00000110'             ; Cursor increments, no display shift.
                CALL    LCD_CMD
LCD_NORM:       MOVLW   B'00001100'             ; Turn the display on.
                GOTO    LCD_CMD                 ; Return via "GOTO".

LCD_RESET:      MOVLW   B'00111000'             ; LCD reset command.
                MOVWF   PORTB                   ; Also sets LCD_E lead high.
                BCF     LCD_E
                GOTO    WAIT_8MS                ; Return via "GOTO".

;*****************************************************************************************
;
;     NAMES:    LCD_CLEAR       Clear the display and home the cursor.
;               LCD_HOME        Home the cursor.
;               LCD_DP          Display a decimal point.
;               LCD_CMD         Send the command byte in W to the LCD.
;               LCD_CHR         Send the data byte in W to the LCD.
;               WAIT_8MS        Do nothing for 8 mS.
;               MS_WAIT         W = The number of mS to wait.
;
;*****************************************************************************************

LCD_CLEAR:      MOVLW   H'01'                   ; Clear the display. Then wait 8mS.
                CALL    LCD_CMD
WAIT_8MS:       MOVLW   8                       ; Set the outer loop to 8mS.
MS_WAIT:        MOVWF   COUNT_0                 ; Save the number of mS delay.
OUTER_LOOP:     CLRW                            ; Set the wait loop to 256.
                CALL    WAIT_LOOP               ; 256 loops x 4 x 1uS = 1.024mS.
                DECFSZ  COUNT_0,F               ; Dec the mS counter.
                GOTO    OUTER_LOOP
                RETURN

LCD_HOME:       MOVLW   H'80'                   ; Position the cursor, line 1 pos 1.
LCD_CMD:        BCF     LCD_RS                  ; Select the instruction register.
                GOTO    LCD_BYTE                ; Send the byte to the LCD.

LCD_DP:         MOVLW   H'A5'                   ; Display a decimal point.
LCD_CHR:        BSF     LCD_RS                  ; Select display data register.
LCD_BYTE:       MOVWF   TEMP_3                  ; Save the byte to send to the LCD.
                CALL    SEND_NIBBLE             ; Send the high nibble to the LCD.
                SWAPF   TEMP_3,F                ; Send the low nibble to the LCD.
SEND_NIBBLE:    MOVLW   B'00001111'
                ANDWF   PORTB,F                 ; Clear the high nibble of port B.
                MOVF    TEMP_3,W                ; Get the byte of data.
                ANDLW   B'11110000'             ; Keep only the high nibble.
                IORWF   PORTB,F                 ; Send the nibble to port B.
                BSF     LCD_E                   ; Pulse the LCD_E lead.
                BCF     LCD_E
WAIT_200US:     MOVLW   256-50                  ; Set the wait loop to 200uS.
WAIT_LOOP:      ADDLW   1                       ; 1 CLK cycles \
                BTFSS   ZERO                    ; 1     "       > 4 CLKs X 1uS
                GOTO    WAIT_LOOP               ; 2     "      /
                RETURN

;*****************************************************************************************
;
;     NAMES:    DISP_RIT        Display 'RIT' on line 1 pos 14. Then display RX or TX.
;               DISP_TX_RX      Display 'RX ' or 'TX ' on line 1 pos 1.
;
;     INPUT:    None.
;
;    OUTPUT:    None.
;
;-----------------------------------------------------------------------------------------
;
;      NAME:    LCD_TEXT
;
;   PURPOSE:    Display on the LCD, the string of characters pointed to by W.
;               The last chr of the string must be a space with the MSB set. (C0)
;
;     INPUT:    W = String offset address. (offset from the base of the text table)
;
;    OUTPUT:    None.
;
;*****************************************************************************************

DISP_RIT:       MOVLW   H'8D'
                CALL    LCD_CMD                 ; Position the cursor, line 1 pos 14.
                MOVLW   RIT_OFF
DISP_TEXT:      CALL    LCD_TEXT                ; Display 'RIT'.
                                                ; Fall through and display RX or TX.
DISP_TX_RX:     CALL    LCD_HOME                ; Goto the start of line 1.
                CALL    CHK_TX                  ; Check the TX button.
                MOVLW   RX                      ; Prepare to display 'RX '.
                BTFSC   PRESSED                 ; Was the TX button pressed ?
                MOVLW   TX                      ; Y. Prepare to display 'TX '.
                                                ; Display the 'TX ' or 'RX '.

LCD_TEXT:       MOVWF   TEMP_1                  ; Save the text start address.
                CALL    TABLE                   ; Get the character
                CALL    LCD_CHR                 ; Display character.
                BTFSC   TEMP_3,3                ; Was it the end of message ? (MSB set)
                RETURN                          ; Y. All done.
                INCF    TEMP_1,W                ; N. Point to next character.
                GOTO    LCD_TEXT                ;    Keep looping until end of message.

;*****************************************************************************************

                END

