package misim.plugins;

import java.awt.*;
import java.awt.event.*;
import misim.*;
import misim.util.*;

public class UnipolarPlugin extends Plugin implements  PortListener,ActionListener 
{
	int portState, ppv, pv;
   	UnipolarCanvas Unipolar;
   	boolean coila,coilb,coilc,coild;    // current coil settings
	boolean pca, pcb, pcc, pcd;         // previous coil positions
   	double grau;
   	boolean flag;
   	String config;
   	double step;
   	TextField tfstep;
   	PortMapper portMapper;
   
   
   	public UnipolarPlugin( Component owner, MachineState machineState, 
                  RobustReaderIF reader, String params ) throws MiSimException
  	{
      	super( owner, machineState, reader, params );
   	}
   	
   	public void init(String params ) throws MiSimException
   	{
      	grau=0;
	coila=false; coilb=false; coilc=false; coild=false;
	pca=false; pcb=false; pcc=false; pcd=false;
      	flag=true;
      	setBackground(Color.white);
      	setLayout(new BorderLayout());
      	setSize(400,400);
     	Unipolar = new UnipolarCanvas();
      	Unipolar.setSize( 250, 250 );
      	setResizable( true );
      	add(Unipolar,BorderLayout.CENTER);
      	step=7.5;
      	tfstep=new TextField("7.5",15);
      	Panel aPanel = new Panel();
      	aPanel.add( new Label( "Step Angle (degrees)" ) );
      	aPanel.add( tfstep );
      	add( aPanel, BorderLayout.SOUTH );
      	tfstep.addActionListener(this);
      	String configErrs = readConfig( params );
      	if( configErrs != null ) throw new MiSimException( Txt.EX_CONFIG_ERROR, configErrs );
      	pack();
   	}

   
   	public void showPlugin(boolean visible)
   	{
      	setVisible( visible );
      	if( visible )
	  machineState.addPortListener( this, portMapper.getPortNames() );
      	else
	  machineState.removePortListener( this );
   	}

   	public int configType() 
   	{
      	return PLUGIN_GUI;
   	}
   
   	public String getConfig()
   	{
	return config;
   	}
   	
   	public String readConfig( String newConfig )
   	{
   	try
	 {
      	 String mapping = newConfig;
      	 PortMapper map = new PortMapper( machineState, mapping, "3210" );
       	 portMapper = map;
       	 config = newConfig;
    	 }
      	catch( MiSimException picex ) 
      	 {
	 return picex.toString();
      	 }
	return null;
   	}
   	
   	
   
   	public String editConfig( String config ) 
   	{
      	return config;
   	}


   	public String getHelp() 
   	{
      	return	"The config must map the four motor signals '3210' to ports.\n"+
		"For instance Port B:3210 maps the four signals onto pins 0-3 of Port A.\n"+
		"Assume that the coils are connected via power transistors and that the "+
		"Common wire is connected to VSS(+)";
   	}  
   
   	public String getAbout()
   	{
	return "Emulate a Unipolar stepper motor. It has 4 port pins(3 2 1 0)";
   	}    

   
   	public void portEvent( int portBits, long clockCount, int PortId)
   	{
	pv=0;
	portMapper.prepare( PortId, portBits );
	coild = portMapper.readBit( '3' );
	coilc = portMapper.readBit( '2' );
	coilb = portMapper.readBit( '1' );
	coila = portMapper.readBit( '0' );
	if(coila==true)pv=pv+1;
	if(coilb==true)pv=pv+2;
	if(coilc==true)pv=pv+4;
	if(coild==true)pv=pv+8;
	Unipolar.paint( Unipolar.getGraphics() ); 
	}
   
   	public void actionPerformed(ActionEvent e)
   	{
	step = Double.valueOf(tfstep.getText()).doubleValue();
   	}
   
   
	public int getBitmask( int portId )
	{
	return portMapper.getBitmask( portId ); 
	}


	class UnipolarCanvas extends Canvas
	{
	public void paint(Graphics g)
	{
	 super.paint(g);
/** 	check previous coil positions and then only look for valid moves
	first check if any coils are on. If none are on ( ie no motor windings 
	are activated )	then the first coil activation's results are indeterminte.
	The program should do a home seek when it starts up. For this simulation, 
	the transition from no energised coils to energised ones will not cause movement.
	Only subsequent moves will work
 */
	switch(ppv)
        {
	case 0: ppv=pv;   // no coils are on
	case 1: if(pv==2)   // coil a was on last
		{
		ppv=pv;
		grau=grau+step;
		}
		else if(pv==8)  		// full step backward
		 {
		 ppv=pv;
		 grau=grau-step;
		 }
		 else if(pv==3)  		// half step foreward
		  {
		  ppv=pv;
		  grau=grau+(step/2);
                  }
		  else if(pv==9)  		//half step backward
		   {
		   ppv=pv;
		   grau=grau-(step/2);
		   }
		break;
	case 2:	if(pv==4)   			// coil b was on last
		{
		ppv=pv;
		grau=grau+step;
		}
		else if(pv==1)   		// full step backward
		 {
		 ppv=pv;
		 grau=grau-step;
		 }
		 else if(pv==6)   		// half step foreward
		  {
		  ppv=pv;
		  grau=grau+(step/2);
                  }
		  else if(pv==3)   		//half step backward
		   {
		   ppv=pv;
 		   grau=grau-(step/2);
		   }
		break;
	case 4:	if(pv==8)   			// coil c was on last
		{
		ppv=pv;
		grau=grau+step;
		}
		else if(pv==2)   		// full step backward
		 {
		 ppv=pv;
		 grau=grau-step;
		 }
		 else if(pv==12)   		// half step foreward
		  {
		  ppv=pv;
		  grau=grau+(step/2);
                  }
		  else if(pv==6)   		//half step backward
		   {
		   ppv=pv;
		   grau=grau-(step/2);
		   }
		break;
	case 8:	if(pv==1)   			// coil d was on last
		{
		ppv=pv;
		grau=grau+step;
		}
		else if(pv==4)   		// full step backward
		 {
		 ppv=pv;
		 grau=grau-step;
		 }
		 else if(pv==9)   		// half step foreward
		  {
		  ppv=pv;
		  grau=grau+(step/2);
                  }
		  else if(pv==12)   		//half step backward
		   {
		   ppv=pv;
		   grau=grau-(step/2);
		   }
		break;
	case 3:	if(pv==2)			// coil ab
		{				// Half Foreward
		ppv=pv;
		grau=grau+(step/2);
		}
		else if(pv==1)			// Half Backward
		 {
		 ppv=pv;
		 grau=grau-(step/2);
		 }
		 else if(pv==6)			// Full foreward
		  {
		  ppv=pv;
		  grau=grau+step;
		  }
		  else if(pv==9)		// Full Backward
		   {
		   ppv=pv;
		   grau=grau-step;
                   }
		break;
	case 6:	if(pv==4)			// coil bc
		{				// Half Foreward
		ppv=pv;
		grau=grau+(step/2);
		}
		else if(pv==2)			// Half Backward
		 {
		 ppv=pv;
		 grau=grau-(step/2);
		 }
		 else if(pv==12)		// Full foreward
		  {
		  ppv=pv;
		  grau=grau+step;
		  }
		  else if(pv==3)		// Full Backward
		   {
		   ppv=pv;
		   grau=grau-step;
                   }
		break;
	case 12: if(pv==8)			// coil cd
		{				// Half Foreward
		ppv=pv;
		grau=grau+(step/2);
		}
		else if(pv==4)			// Half Backward
		 {
		 ppv=pv;
		 grau=grau-(step/2);
		 }
		 else if(pv==9)			// Full foreward
		  {
		  ppv=pv;
		  grau=grau+step;
		  }
		  else if(pv==6)		// Full Backward
		   {
		   ppv=pv;
		   grau=grau-step;
                   }
		break;
	case 9:	if(pv==1)			// coil da
		{				// Half Foreward
		ppv=pv;
		grau=grau+(step/2);
		}
		else if(pv==8)			// Half Backward
		 {
		 ppv=pv;
		 grau=grau-(step/2);
		 }
		 else if(pv==3)			// Full foreward
		  {
		  ppv=pv;
		  grau=grau+step;
		  }
		  else if(pv==12)		// Full Backward
		   {
		   ppv=pv;
		   grau=grau-step;
                   }
		break;
	} 
        if((grau==360) || (grau==-360))grau=0;
        g.setColor(Color.blue);
        g.drawOval(30,30,190,190);
      	g.setColor(Color.red);
      	double angle = grau*Math.PI/180;
      	int x1 = 125+(int)Math.round((95*Math.sin(angle)));
      	int y1 = 125-(int)Math.round((95*Math.cos(angle)));
      	int x2 = 125-(int)Math.round((95*Math.sin(angle)));
      	int y2 = 125+(int)Math.round((95*Math.cos(angle)));

      	int dx = (int)Math.round(4*Math.cos(angle));
      	int dy = (int)Math.round(4*Math.sin(angle));

      	int[] xpoints = new int[] {x1-dx, x1+dx, x2+dx, x2-dx};
      	int[] ypoints = new int[] {y1-dy, y1+dy, y2+dy, y2-dy};
      	
      	g.fillPolygon( xpoints, ypoints, 4 );
      	g.setColor(Color.green.darker());
      	g.fillOval( x1-4, y1-4, 8, 8 );
      }
   }
}




